#!/usr/bin/python
# Copyright 2013, 2015, 2016, 2017, Cumulus Networks, Inc.  All rights reserved.

import shlex
import errno
import imp
import os
import sys

import cumulus

PLATFORM_DIR=os.path.dirname(__file__)

_TEST = ('BOOT_IMAGE=/vmlinuz-4.1.0-cl-1-amd64 '
         'root=UUID=5b15d39a-0e0b-4186-ae19-f30f7c4616d7 '
         'ro '
         'cl_platform=unknown '
         'console=ttyS1,115200n8 '
         'quiet',

         'BOOT_IMAGE=/vmlinuz-4.1.0-cl-1-amd64 '
         'root=UUID=e9edff93-c7fd-4b92-a8ac-ea0ba75ed17f '
         'ro '
         'cl_platform=dell_s4000_c2338 '
         'console=ttyS0,115200n8 '
         'quiet',

         # with some quoted strings
         'BOOT_IMAGE=/vmlinuz-4.1.0-cl-1-amd64 '
         'root=UUID=e9edff93-c7fd-4b92-a8ac-ea0ba75ed17f '
         'ro '
         'console=ttyS0,115200n8 '
         'quiet '
         '42="the answer" '
         'foo="foo bar" '
         'cl_platform=dell_s4000_c2338'
        )

def _parse_cmdline(cmdline=None):
    if cmdline is None:
        cmdline = open('/proc/cmdline', 'r').read()

    args = shlex.split(cmdline)
    for arg in args:
        if arg.startswith('cl_platform'):
            platform = arg[12:]
            return platform.replace('_', ',', 1)

    return 'unknown'

def _get_platform():
    try:
        return _parse_cmdline()
    except:
        return 'unknown'

class NoSuchPlatform(Exception):
    pass

def _get_platform_module(platform):
    module_name = platform.replace(',', '_', 1)

    try:
        module = imp.find_module(module_name, [PLATFORM_DIR])
    except ImportError, e:
        # if we can't find the platform specific module, use the ACPI
        # platform.
        try:
            module = imp.find_module("cumulus_acpi", [PLATFORM_DIR])
        except ImportError, e:
            raise NoSuchPlatform('platform not found in %s: %s' %
                                 (PLATFORM_DIR, str(e)))

    loaded = imp.load_module('loaded_platform', *module)

    if hasattr(loaded, 'Platform'):
        if loaded.Platform.name == 'cumulus,acpi' and loaded.Platform.probe() is False:
            # Not an ACPI capable platform.  Fallback to cumulus_generic
            try:
                module = imp.find_module("cumulus_generic", [PLATFORM_DIR])
            except ImportError, e:
                raise NoSuchPlatform('platform not found in %s: %s' %
                                     (PLATFORM_DIR, str(e)))
            loaded = imp.load_module('loaded_platform', *module)
        return loaded.Platform
    else:
        raise NoSuchPlatform('module defines no Platform class: %s'
                             % str(module))

_loaded = {}
def probe(forced_platform=None, reprobe=False, use_acpi=False):
    global _loaded

    if forced_platform is not None:
        platform = forced_platform
    else:
        platform = _get_platform()

    if use_acpi and 'unknown' in platform:
        platform = 'cumulus,acpi'

    if platform not in _loaded.keys() or reprobe:
        module = _get_platform_module(platform)
        _loaded[platform] = module

    return _loaded[platform]()

if __name__ == '__main__':
    print 'current cmdline platform:', _parse_cmdline()
    print 'test[0] unknown:', _parse_cmdline(_TEST[0]) == 'unknown'
    print 'test[1] dell,s4000_c2338:', _parse_cmdline(_TEST[1]) == 'dell,s4000_c2338'
    print 'test[2] dell,s4000_c2338:', _parse_cmdline(_TEST[2]) == 'dell,s4000_c2338'
