#
# Copyright 2018.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors


class AcctonAS7726QSFP28(cumulus.platform.TridentThree100GPort):
    def __init__(self, label, serdes, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.TridentThree100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   tx_lane_map=tx_lane_map,
                                                   rx_lane_map=rx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=None,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

class AcctonAS7726_32XSwitch(cumulus.platform.BCMSwitch):
    _qsfp28 = AcctonAS7726QSFP28
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _qsfp28(label="1",  serdes=(0, 1, 2, 3),
                  tx_lane_map=0x3210, rx_lane_map=0x1320, tx_polarity_flip=0xa,
                  rx_polarity_flip=0xc,),
        _qsfp28(label="2",  serdes=(4, 5, 6, 7),
                  tx_lane_map=0x3210, rx_lane_map=0x0123, tx_polarity_flip=0xa,
                  rx_polarity_flip=0xa,),
        _qsfp28(label="3",  serdes=(8, 9, 10, 11),
                  tx_lane_map=0x3210, rx_lane_map=0x0123, tx_polarity_flip=0x5,
                  rx_polarity_flip=0xb,),
        _qsfp28(label="4",  serdes=(12, 13, 14, 15),
                  tx_lane_map=0x3210, rx_lane_map=0x0123, tx_polarity_flip=0x5,
                  rx_polarity_flip=0x5,),
        _qsfp28(label="5",  serdes=(16, 17, 18, 19),
                  tx_lane_map=0x3210, rx_lane_map=0x0123, tx_polarity_flip=0x5,
                  rx_polarity_flip=0xa,),
        _qsfp28(label="6",  serdes=(20, 21, 22, 23),
                  tx_lane_map=0x3210, rx_lane_map=0x0123, tx_polarity_flip=0x5,
                  rx_polarity_flip=0x5,),
        _qsfp28(label="7",  serdes=(24, 25, 26, 27),
                  tx_lane_map=0x3210, rx_lane_map=0x0123, tx_polarity_flip=0x5,
                  rx_polarity_flip=0x9,),
        _qsfp28(label="8",  serdes=(28, 29, 30, 31),
                  tx_lane_map=0x3120, rx_lane_map=0x1230, tx_polarity_flip=0x3,
                  rx_polarity_flip=0xa,),
        _qsfp28(label="9",  serdes=(32, 33, 34, 35),
                  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x5,
                  rx_polarity_flip=0x5,),
        _qsfp28(label="10", serdes=(36, 37, 38, 39),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x5,
                  rx_polarity_flip=0xa,),
        _qsfp28(label="11", serdes=(40, 41, 42, 43),
                  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x5,
                  rx_polarity_flip=0x5,),
        _qsfp28(label="12", serdes=(44, 45, 46, 47),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x5,
                  rx_polarity_flip=0xa,),
        _qsfp28(label="13", serdes=(48, 49, 50, 51),
                  tx_lane_map=0x2310, rx_lane_map=0x0213, tx_polarity_flip=0x9,
                  rx_polarity_flip=0x3,),
        _qsfp28(label="14", serdes=(52, 53, 54, 55),
                  tx_lane_map=0x3210, rx_lane_map=0x1320, tx_polarity_flip=0xa,
                  rx_polarity_flip=0x8,),
        _qsfp28(label="15", serdes=(56, 57, 58, 59),
                  tx_lane_map=0x1230, rx_lane_map=0x0123, tx_polarity_flip=0x7,
                  rx_polarity_flip=0xa,),
        _qsfp28(label="16", serdes=(60, 61, 62, 63),
                  tx_lane_map=0x1230, rx_lane_map=0x0123, tx_polarity_flip=0x5,
                  rx_polarity_flip=0xf,),
        _qsfp28(label="17", serdes=(64, 65, 66, 67),
                  tx_lane_map=0x1302, rx_lane_map=0x3210, tx_polarity_flip=0x3,
                  rx_polarity_flip=0xa,),
        _qsfp28(label="18", serdes=(68, 69, 70, 71),
                  tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=0xa,
                  rx_polarity_flip=0x5,),
        _qsfp28(label="19", serdes=(72, 73, 74, 75),
                  tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=0xb,
                  rx_polarity_flip=0xa,),
        _qsfp28(label="20", serdes=(76, 77, 78, 79),
                  tx_lane_map=0x2013, rx_lane_map=0x3201, tx_polarity_flip=0x3,
                  rx_polarity_flip=0x2,),
        _qsfp28(label="21", serdes=(80, 81, 82, 83),
                  tx_lane_map=0x3210, rx_lane_map=0x3120, tx_polarity_flip=0x5,
                  rx_polarity_flip=0xc,),
        _qsfp28(label="22", serdes=(84, 85, 86, 87),
                  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x5,
                  rx_polarity_flip=0x5,),
        _qsfp28(label="23", serdes=(88, 89, 90, 91),
                  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x5,
                  rx_polarity_flip=0xa,),
        _qsfp28(label="24", serdes=(92, 93, 94, 95),
                  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x5,
                  rx_polarity_flip=0xa,),
        _qsfp28(label="25", serdes=(96, 97, 98, 99),
                  tx_lane_map=0x0123, rx_lane_map=0x3012, tx_polarity_flip=0x5,
                  rx_polarity_flip=0x0,),
        _qsfp28(label="26", serdes=(100, 101, 102, 103),
                  tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0x5,
                  rx_polarity_flip=0x6,),
        _qsfp28(label="27", serdes=(104, 105, 106, 107),
                  tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0x5,
                  rx_polarity_flip=0x5,),
        _qsfp28(label="28", serdes=(108, 109, 110, 111),
                  tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0x5,
                  rx_polarity_flip=0xa,),
        _qsfp28(label="29", serdes=(112, 113, 114, 115),
                  tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0x5,
                  rx_polarity_flip=0x5,),
        _qsfp28(label="30", serdes=(116, 117, 118, 119),
                  tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0x5,
                  rx_polarity_flip=0xa,),
        _qsfp28(label="31", serdes=(120, 121, 122, 123),
                  tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0x5,
                  rx_polarity_flip=0x5,),
        _qsfp28(label="32", serdes=(124, 125, 126, 127),
                  tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0x2,
                  rx_polarity_flip=0x2,),
    )

class AcctonAS7726_32XCPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/platform/accton_as7726_32x_cpld.0'
    
class AcctonAS7726_32XTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = 'Temp sensor behind networking asic'
    driver_path = '/sys/bus/i2c/devices/26-0048'
    temp_fan_min  = 51
    temp_fan_max  = 58
    temp_max  = 66
    temp_crit = 75 

class AcctonAS7726_32XTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'Temp sensor in front of networking asic'
    driver_path = '/sys/bus/i2c/devices/26-0049'
    temp_fan_min  = 62
    temp_fan_max  = 66
    temp_max  = 72
    temp_crit = 80

class AcctonAS7726_32XTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = 'Temp sensor front left'
    driver_path = '/sys/bus/i2c/devices/26-004a'
    temp_fan_min  = 41
    temp_fan_max  = 54
    temp_max  = 58
    temp_crit = 66 

class AcctonAS7726_32XTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    driver_path = '/sys/bus/i2c/devices/26-004b'
    description = 'Temp Sensor Near CPU'
    temp_fan_min = 37 
    temp_fan_max = 50
    temp_max     = 57
    temp_crit    = 65

AcctonAS7726_32X_core_path = '/sys/devices/platform/coretemp.0'

class AcctonAS7726_32XCPUTemp(cumulus.sensors.Temp_Unit):
    description = 'Intel CPU die sensor'
    driver_path = AcctonAS7726_32X_core_path
    temp_fan_min = 58  
    temp_fan_max = 65
    temp_max     = 70
    temp_crit    = 98

class AcctonAS7726_32XTemp5(AcctonAS7726_32XCPUTemp):
    name = 'Temp5'
    driver_hwmon = ['temp2']

class AcctonAS7726_32XTemp6(AcctonAS7726_32XCPUTemp):
    name = 'Temp6'
    driver_hwmon = ['temp3']

class AcctonAS7726_32XTemp7(AcctonAS7726_32XCPUTemp):
    name = 'Temp7'
    driver_hwmon = ['temp4']

class AcctonAS7726_32XTemp8(AcctonAS7726_32XCPUTemp):
    name = 'Temp8'
    driver_path = AcctonAS7726_32X_core_path
    driver_hwmon = ['temp5']

AcctonAS7726_32X_PSU1_i2c_path = '/sys/bus/i2c/devices/21-005b'
AcctonAS7726_32X_PSU2_i2c_path = '/sys/bus/i2c/devices/20-0058'

class AcctonAS7726_32XPSUTemp(cumulus.sensors.Temp_Unit):
    temp_crit = 60
    temp_max = 50
    temp_fan_min = 25
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

class AcctonAS7726_32XTemp9(AcctonAS7726_32XPSUTemp):
    psu = 'PSU1'
    name = psu + 'Temp1'
    description = psu + ' Inlet Temp Sensor'
    driver_path = AcctonAS7726_32X_PSU1_i2c_path
    driver_hwmon = ['temp1']

class AcctonAS7726_32XTemp10(AcctonAS7726_32XPSUTemp):
    psu = 'PSU1'
    name = psu + 'Temp2'
    description = psu + ' Max Temp Sensor'
    driver_path = AcctonAS7726_32X_PSU1_i2c_path
    driver_hwmon = ['temp2']

class AcctonAS7726_32XTemp11(AcctonAS7726_32XPSUTemp):
    psu = 'PSU2'
    name = psu + 'Temp1'
    description = psu + ' Inlet Temp Sensor'
    driver_path = AcctonAS7726_32X_PSU2_i2c_path
    driver_hwmon = ['temp1']

class AcctonAS7726_32XTemp12(AcctonAS7726_32XPSUTemp):
    psu = 'PSU2'
    name = psu + 'Temp2'
    description = psu + ' Max Temp Sensor'
    driver_path = AcctonAS7726_32X_PSU2_i2c_path
    driver_hwmon = ['temp2']    

class AcctonAS7726_32X_ASICTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_fan_min = 77
    temp_fan_max = 84
    temp_max     = 88
    temp_crit    = 96

class AcctonAS7726_32XTemp13(AcctonAS7726_32X_ASICTemp):
    name = 'Temp9'
    driver_hwmon = ['temp1']

class AcctonAS7726_32XTemp14(AcctonAS7726_32X_ASICTemp):
    name = 'Temp10'
    driver_hwmon = ['temp2']

class AcctonAS7726_32XTemp15(AcctonAS7726_32X_ASICTemp):
    name = 'Temp11'
    driver_hwmon = ['temp3']

class AcctonAS7726_32XTemp16(AcctonAS7726_32X_ASICTemp):
    name = 'Temp12'
    driver_hwmon = ['temp4']

class AcctonAS7726_32XTemp17(AcctonAS7726_32X_ASICTemp):
    name = 'Temp13'
    driver_hwmon = ['temp5']

class AcctonAS7726_32XTemp18(AcctonAS7726_32X_ASICTemp):
    name = 'Temp14'
    driver_hwmon = ['temp6']

class AcctonAS7726_32XTemp19(AcctonAS7726_32X_ASICTemp):
    name = 'Temp15'
    driver_hwmon = ['temp7']

class AcctonAS7726_32XTemp20(AcctonAS7726_32X_ASICTemp):
    name = 'Temp16'
    driver_hwmon = ['temp8']

class AcctonAS7726_32X_PSU(cumulus.sensors.PSU_Unit):
    init_done = False
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            cmd = ['/sbin/rmmod', 'cpr4011']
            retrm =  subprocess.call(cmd)
            if retrm == 0:
                cmd = ['/sbin/modprobe', 'cpr4011']
                subprocess.call(cmd)
            self.init_done = True

class AcctonAS7726_32XPSU1(AcctonAS7726_32X_PSU):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class AcctonAS7726_32XPSU2(AcctonAS7726_32X_PSU):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

class AcctonAS7726_32XFanBase(cumulus.sensors.Fan_Unit):
    hwmon_search = False
    driver_path = AcctonAS7726_32XCPLD.path
    # The minimum value that the
    # pwm register can hold is 4. Mapping 
    # that to 0~255 range by multiplying 
    # by 17.
    minpwm  = 119
    # The CPLD driver only exports "pwm1", which controls all the
    # fans.
    pwm_hwmon = ['pwm1']

# 7726 has 6 fan trays, each of which has a front
# and a rear fan.
class AcctonAS7726_32XFan(AcctonAS7726_32XFanBase):
    fan_min = 12000
    fan_max = 21300

class AcctonAS7726_32XFanR(AcctonAS7726_32XFanBase):
    fan_min = 10100
    fan_max = 17700

class AcctonAS7726_32XFan1(AcctonAS7726_32XFan):
    name = 'Fan1'
    description = 'Fan Tray 1, Fan A'
    # driver is fan speed ...
    driver_hwmon = ['fan1']
    # status is "cpld_hwmon"
    cpld_hwmon = ['fan_1']


class AcctonAS7726_32XFan2(AcctonAS7726_32XFanR):
    name = 'Fan2'
    description = 'Fan Tray 1, Fan B'
    driver_hwmon = ['fan2']
    cpld_hwmon = ['fan_1']

class AcctonAS7726_32XFan3(AcctonAS7726_32XFan):
    name = 'Fan3'
    description = 'Fan Tray 2, Fan A'
    driver_hwmon = ['fan3']
    cpld_hwmon = ['fan_2']

class AcctonAS7726_32XFan4(AcctonAS7726_32XFanR):
    name = 'Fan4'
    description = 'Fan Tray 2, Fan B'
    driver_hwmon = ['fan4']
    cpld_hwmon = ['fan_2']

class AcctonAS7726_32XFan5(AcctonAS7726_32XFan):
    name = 'Fan5'
    description = 'Fan Tray 3, Fan A'
    driver_hwmon = ['fan5']
    cpld_hwmon = ['fan_3']

class AcctonAS7726_32XFan6(AcctonAS7726_32XFanR):
    name = 'Fan6'
    description = 'Fan Tray 3, Fan B'
    driver_hwmon = ['fan6']
    cpld_hwmon = ['fan_3']

class AcctonAS7726_32XFan7(AcctonAS7726_32XFan):
    name = 'Fan7'
    description = 'Fan Tray 4, Fan A'
    driver_hwmon = ['fan7']
    cpld_hwmon = ['fan_4']

class AcctonAS7726_32XFan8(AcctonAS7726_32XFanR):
    name = 'Fan8'
    description = 'Fan Tray 4, Fan B'
    driver_hwmon = ['fan8']
    cpld_hwmon = ['fan_4']

class AcctonAS7726_32XFan9(AcctonAS7726_32XFan):
    name = 'Fan9'
    description = 'Fan Tray 5, Fan A'
    driver_hwmon = ['fan9']
    cpld_hwmon = ['fan_5']

class AcctonAS7726_32XFan10(AcctonAS7726_32XFanR):
    name = 'Fan10'
    description = 'Fan Tray 5, Fan B'
    driver_hwmon = ['fan10']
    cpld_hwmon = ['fan_5']

class AcctonAS7726_32XFan11(AcctonAS7726_32XFan):
    name = 'Fan11'
    description = 'Fan Tray 6, Fan A'
    driver_hwmon = ['fan11']
    cpld_hwmon = ['fan_6']

class AcctonAS7726_32XFan12(AcctonAS7726_32XFanR):
    name = 'Fan12'
    description = 'Fan Tray 6, Fan B'
    driver_hwmon = ['fan12']
    cpld_hwmon = ['fan_6']

class AcctonAS7726SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'Diag'
    led_location = 'led_diag'
    bad_led_color = 'red'

class AcctonAS7726StatusLEDs():
    units = (AcctonAS7726SystemLED(),
             )

class AcctonAS7726_32XSensors():
    units = ( AcctonAS7726_32XTemp1(),
              AcctonAS7726_32XTemp2(),
              AcctonAS7726_32XTemp3(),
              AcctonAS7726_32XTemp4(),
              AcctonAS7726_32XTemp5(),
              AcctonAS7726_32XTemp6(),
              AcctonAS7726_32XTemp7(),
              AcctonAS7726_32XTemp8(),
              AcctonAS7726_32XTemp9(),
              AcctonAS7726_32XTemp10(),
              AcctonAS7726_32XTemp11(),
              AcctonAS7726_32XTemp12(),
              AcctonAS7726_32XTemp13(),
              AcctonAS7726_32XTemp14(),
              AcctonAS7726_32XTemp15(),
              AcctonAS7726_32XTemp16(),
              AcctonAS7726_32XTemp17(),
              AcctonAS7726_32XTemp18(),
              AcctonAS7726_32XTemp19(),
              AcctonAS7726_32XTemp20(),
              AcctonAS7726_32XPSU1(),
              AcctonAS7726_32XPSU2(),
              AcctonAS7726_32XFan1(),
              AcctonAS7726_32XFan2(),
              AcctonAS7726_32XFan3(),
              AcctonAS7726_32XFan4(),
              AcctonAS7726_32XFan5(),
              AcctonAS7726_32XFan6(),
              AcctonAS7726_32XFan7(),
              AcctonAS7726_32XFan8(),
              AcctonAS7726_32XFan9(),
              AcctonAS7726_32XFan10(),
              AcctonAS7726_32XFan11(),
              AcctonAS7726_32XFan12(),
   )

class AcctonAS7726_32XPWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (AcctonAS7726_32XFan1(),
            AcctonAS7726_32XFan2(),
            AcctonAS7726_32XFan3(),
            AcctonAS7726_32XFan4(),
            AcctonAS7726_32XFan5(),
            AcctonAS7726_32XFan6(),
            AcctonAS7726_32XFan7(),
            AcctonAS7726_32XFan8(),
            AcctonAS7726_32XFan9(),
            AcctonAS7726_32XFan10(),
            AcctonAS7726_32XFan11(),
            AcctonAS7726_32XFan12(),
            )
    temps = (AcctonAS7726_32XTemp1(),
             AcctonAS7726_32XTemp2(),
             AcctonAS7726_32XTemp3(),
             AcctonAS7726_32XTemp4(),
             AcctonAS7726_32XTemp5(),
             AcctonAS7726_32XTemp6(),
             AcctonAS7726_32XTemp7(),
             AcctonAS7726_32XTemp8(),
             AcctonAS7726_32XTemp9(),
             AcctonAS7726_32XTemp10(),
             AcctonAS7726_32XTemp11(),
             AcctonAS7726_32XTemp12(),
             AcctonAS7726_32XTemp13(),
             AcctonAS7726_32XTemp14(),
             AcctonAS7726_32XTemp15(),
             AcctonAS7726_32XTemp16(),
             AcctonAS7726_32XTemp17(),
             AcctonAS7726_32XTemp18(),
             AcctonAS7726_32XTemp19(),
             AcctonAS7726_32XTemp20(),
             )

class AcctonAS7726_32XPWMs():
    units = (AcctonAS7726_32XPWM1(),
             )

class AcctonAS7726_32X(cumulus.platform.Platform):
    name = 'accton,as7726_32x'
    chip = cumulus.platform.TridentThreeX7_56870_Chip()
    switch = AcctonAS7726_32XSwitch(chip)
    cpld = AcctonAS7726_32XCPLD()
    statusleds = AcctonAS7726StatusLEDs()
    pwms = AcctonAS7726_32XPWMs()
    sensors = AcctonAS7726_32XSensors()

Platform = AcctonAS7726_32X
