#
# Copyright (c) 2019.  Cumulus Networks, Inc.  All rights reserved.
#

import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
from cumulus.platforms.cel_sea2que2 import *

class CelQuestone2SFP28(cumulus.platform.TridentThree25GPort):

    def __init__(self, label, serdes, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp28')
        cumulus.platform.TridentThree25GPort.__init__(self,
                                                  label=label,
                                                  serdes=serdes,
                                                  leds=None,
                                                  phy_types=None,
                                                  phy_addrs=None,
                                                  phy_layer=sfp.type.upper(),
                                                  tx_lane_map=tx_lane_map,
                                                  rx_lane_map=rx_lane_map,
                                                  tx_polarity_flip=tx_polarity_flip,
                                                  rx_polarity_flip=rx_polarity_flip,
                                                  pre_emphasis=pre_emphasis,
                                                  unit=0,
                                                  oversub=True)
        self.sfp = sfp

class CelQuestone2QSFP28(cumulus.platform.TridentThree100GPort):

    def __init__(self, label, serdes, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.TridentThree100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   leds=None,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

# Celestica Questone2 Switch Ports

class CelQuestone2Switch(cumulus.platform.BCMSwitch):
    _sfp28 = CelQuestone2SFP28
    _qsfp28 = CelQuestone2QSFP28
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (

        # TSCF12
        _sfp28(label="1",  serdes=( 48,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="2",  serdes=( 49,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True) ,
        _sfp28(label="3",  serdes=( 50,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="4",  serdes=( 51,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True) ,

        # TSCF14
        _sfp28(label="5",  serdes=( 56,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="6",  serdes=( 57,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True) ,
        _sfp28(label="7",  serdes=( 58,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="8",  serdes=( 59,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True) ,

        # TSCF15
        _sfp28(label="9",  serdes=( 60,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True) ,
        _sfp28(label="10", serdes=( 61,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="11", serdes=( 62,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True) ,
        _sfp28(label="12", serdes=( 63,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCF19
        _sfp28(label="13", serdes=( 76,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True) ,
        _sfp28(label="14", serdes=( 77,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="15", serdes=( 78,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True) ,
        _sfp28(label="16", serdes=( 79,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF21
        _sfp28(label="17", serdes=( 84,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True) ,
        _sfp28(label="18", serdes=( 85,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="19", serdes=( 86,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True) ,
        _sfp28(label="20", serdes=( 87,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF23
        _sfp28(label="21", serdes=( 92,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True) ,
        _sfp28(label="22", serdes=( 93,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="23", serdes=( 94,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True) ,
        _sfp28(label="24", serdes=( 95,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF3
        _sfp28(label="25", serdes=( 12,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True) ,
        _sfp28(label="26", serdes=( 13,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="27", serdes=( 14,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True) ,
        _sfp28(label="28", serdes=( 15,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF5
        _sfp28(label="29", serdes=( 20,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True) ,
        _sfp28(label="30", serdes=( 21,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="31", serdes=( 22,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True) ,
        _sfp28(label="32", serdes=( 23,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF7
        _sfp28(label="33", serdes=( 28,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True) ,
        _sfp28(label="34", serdes=( 29,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True) ,
        _sfp28(label="35", serdes=( 30,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True) ,
        _sfp28(label="36", serdes=( 31,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF24
        _sfp28(label="37", serdes=( 96,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True) ,
        _sfp28(label="38", serdes=( 97,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="39", serdes=( 98,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True) ,
        _sfp28(label="40", serdes=( 99,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCF26
        _sfp28(label="41", serdes=(104,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="42", serdes=(105,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True) ,
        _sfp28(label="43", serdes=(106,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="44", serdes=(107,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True) ,

        # TSCF28
        _sfp28(label="45", serdes=(112,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="46", serdes=(113,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True) ,
        _sfp28(label="47", serdes=(114,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="48", serdes=(115,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True) ,

        # TSCF16 (spec says TSCF0)
        _qsfp28(label="49", serdes=( 64,  65,  66,  67), tx_lane_map=0x1302, rx_lane_map=0x1023, tx_polarity_flip=0x000d, rx_polarity_flip=0x0000),

        # TSCF17 (spec says TSCF8)
        _qsfp28(label="50", serdes=( 68,  69,  70,  71), tx_lane_map=0x1032, rx_lane_map=0x0213, tx_polarity_flip=0x000a, rx_polarity_flip=0x0009),

        # TSCF30 (spec says TSCF1)
        _qsfp28(label="51", serdes=(120, 121, 122, 123), tx_lane_map=0x2301, rx_lane_map=0x2031, tx_polarity_flip=0x000a, rx_polarity_flip=0x0001),

        # TSCF31 (spec says TSCF10)
        _qsfp28(label="52", serdes=(124, 125, 126, 127), tx_lane_map=0x0132, rx_lane_map=0x3012, tx_polarity_flip=0x0003, rx_polarity_flip=0x0002),

        # TSCF0 (spec says TSCF16)
        _qsfp28(label="53", serdes=(  0,   1,   2,   3), tx_lane_map=0x0123, rx_lane_map=0x1302, tx_polarity_flip=0x0004, rx_polarity_flip=0x000e),

        # TSCF8 (spec says TSCF17)
        _qsfp28(label="54", serdes=( 32,  33,  34,  35), tx_lane_map=0x1302, rx_lane_map=0x1032, tx_polarity_flip=0x0007, rx_polarity_flip=0x0002),

        # TSCF1 (spec says TSCF30)
        _qsfp28(label="55", serdes=(  4,   5,   6,   7), tx_lane_map=0x1032, rx_lane_map=0x1302, tx_polarity_flip=0x000e, rx_polarity_flip=0x0001),

        # TSCF10 (spec says TSCF31)
        _qsfp28(label="56", serdes=( 40,  41,  42,  43), tx_lane_map=0x3120, rx_lane_map=0x1032, tx_polarity_flip=0x0006, rx_polarity_flip=0x0002),

    )

# Intel Denverton CPU

#class CelSea2Que2CoreTemp(cumulus.sensors.Temp_Unit):
#    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
#    driver_path = '/sys/devices/platform/coretemp.0'
#    no_shutdown = True
#    temp_crit = 95
#    temp_max = 75
#
#class CelSea2Que2CPUTemp1(CelSea2Que2CoreTemp):
#    name = 'Temp22'
#    description = 'Physical id 0'
#    driver_hwmon = ['temp1']
#
#class CelSea2Que2CPUTemp2(CelSea2Que2CoreTemp):
#    name = 'Temp23'
#    description = 'Core 2'
#    driver_hwmon = ['temp4']
#
#class CelSea2Que2CPUTemp3(CelSea2Que2CoreTemp):
#    name = 'Temp24'
#    description = 'Core 4'
#    driver_hwmon = ['temp6']
#
#class CelSea2Que2CPUTemp4(CelSea2Que2CoreTemp):
#    name = 'Temp25'
#    description = 'Core 8'
#    driver_hwmon = ['temp10']
#
#class CelSea2Que2CPUTemp5(CelSea2Que2CoreTemp):
#    name = 'Temp26'
#    description = 'Core 14'
#    driver_hwmon = ['temp16']

# Sensors class

class CelQuestone2Sensors():
    units = (CelSea2Que2ASICTemp1(),
             CelSea2Que2ASICTemp2(),
             CelSea2Que2ASICTemp3(),
             CelSea2Que2ASICTemp4(),
             CelSea2Que2ASICTemp5(),
             CelSea2Que2ASICTemp6(),
             CelSea2Que2ASICTemp7(),
             CelSea2Que2ASICTemp8(),
             CelSea2Que2ASICTemp9(),
             CelSea2Que2ASICTemp10(),
             CelSea2Que2ASICTemp11(),
             CelSea2Que2ASICTemp12(),
             CelSea2Que2BMCTemp1(),
             CelSea2Que2BMCTemp2(),
             CelSea2Que2BMCTemp3(),
             CelSea2Que2BMCTemp4(),
             CelSea2Que2BMCTemp5(),
             CelSea2Que2BMCTemp6(),
             CelSea2Que2BMCTemp7(),
             CelSea2Que2BMCTemp8(),
             CelSea2Que2BMCTemp9(),
             CelSea2Que2Fan1(),
             CelSea2Que2Fan2(),
             CelSea2Que2Fan3(),
             CelSea2Que2Fan4(),
             CelSea2Que2Fan5(),
             CelSea2Que2Fan6(),
             CelSea2Que2Fan7(),
             CelSea2Que2Fan8(),
             CelSea2Que2PSU1(),
             CelSea2Que2PSU2(),
             CelSea2Que2PSU1Temp1(),
             CelSea2Que2PSU1Temp2(),
             CelSea2Que2PSU1Fan1(),
             CelSea2Que2PSU2Temp1(),
             CelSea2Que2PSU2Temp2(),
             CelSea2Que2PSU2Fan1(),
             )

class CelQuestone2(cumulus.platform.Platform):
    name = 'cel,questone_2'
    chip = cumulus.platform.TridentThreeX7_56873_Chip()
    switch = CelQuestone2Switch(chip)
    bmc = CelSea2Que2BMC()
    sensors = CelQuestone2Sensors()
    statusleds = None
    pwms = None

Platform = CelQuestone2
