#
# Copyright 2017.  Cumulus Networks, Inc.
#
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
from cumulus.platforms.cel_redstone_xp import CelRedstoneXpSwitch

vhwmon_path = '/sys/class/misc/vhwmon/bmc:'

class CelRedstoneXpbCoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_max = 82
    temp_crit = 104
    no_shutdown = True

class CelRedstoneXpbTemp1(CelRedstoneXpbCoreTemp):
    name = 'Temp1'
    description = 'Physical id 0'
    driver_hwmon = ['temp1']

class CelRedstoneXpbTemp2(CelRedstoneXpbCoreTemp):
    name = 'Temp2'
    description = 'Core 0'
    driver_hwmon = ['temp2']

class CelRedstoneXpbTemp3(CelRedstoneXpbCoreTemp):
    name = 'Temp3'
    description = 'Core 1'
    driver_hwmon = ['temp3']

class CelRedstoneXpbSysTemp(cumulus.sensors.Temp_Unit):
    driver_path = vhwmon_path + 'temp'

class CelRedstoneXpbTemp4(CelRedstoneXpbSysTemp):
    name = 'Temp4'
    description = 'Base_Temp_U5'
    temp_crit = 100
    temp_max = 95
    driver_hwmon = ['temp1']

class CelRedstoneXpbTemp5(CelRedstoneXpbSysTemp):
    name = 'Temp5'
    description = 'Base_Temp_U7'
    temp_crit = 55
    temp_max = 50
    driver_hwmon = ['temp2']

class CelRedstoneXpbTemp6(CelRedstoneXpbSysTemp):
    name = 'Temp6'
    description = 'Switch_U26_temp'
    temp_crit = 55
    temp_max = 50
    driver_hwmon = ['temp3']

class CelRedstoneXpbTemp7(CelRedstoneXpbSysTemp):
    name = 'Temp7'
    description = 'Switch_U13_temp'
    temp_crit = 55
    temp_max = 50
    driver_hwmon = ['temp4']

class CelRedstoneXpbTemp8(CelRedstoneXpbSysTemp):
    name = 'Temp8'
    description = 'TempCOM-E'
    temp_crit = 95
    temp_max = 90
    driver_hwmon = ['temp5']

class CelRedstoneXpbTemp9(CelRedstoneXpbSysTemp):
    name = 'Temp9'
    description = 'RXP_Temp_U8'
    temp_crit = 80
    temp_max = 74
    driver_hwmon = ['temp6']

class CelRedstoneXpbTemp10(CelRedstoneXpbSysTemp):
    name = 'Temp10'
    description = 'RXP_Temp_U2'
    temp_crit = 80
    temp_max = 74
    driver_hwmon = ['temp7']

class CelRedstoneXpbTemp11(CelRedstoneXpbSysTemp):
    name = 'Temp11'
    description = 'RXP_Temp_U23'
    temp_crit = 55
    temp_max = 50
    driver_hwmon = ['temp8']

class CelRedstoneXpbTemp12(CelRedstoneXpbSysTemp):
    name = 'Temp12'
    description = 'RXP_Temp_U22'
    temp_crit = 55
    temp_max = 50
    driver_hwmon = ['temp9']

class CelRedstoneXpb_PSU(cumulus.sensors.PSU_Unit):
    driver_hwmon = ['power1']

class CelRedstoneXpb_PSU1(CelRedstoneXpb_PSU):
    name = 'PSU1'
    driver_path = vhwmon_path + 'psul'
    psu_ok = 0

    def get_status(self):
        val = self.attrs.get('power1_input', 0)
        if val != 0:
            val = 1
        self.attrs['psu_pwr1_all_ok'] = val
        CelRedstoneXpb_PSU1.psu_ok = val
        return cumulus.sensors.PSU_Unit.get_status(self)

class CelRedstoneXpb_PSU2(CelRedstoneXpb_PSU):
    name = 'PSU2'
    driver_path = vhwmon_path + 'psur'
    psu_ok = 0

    def get_status(self):
        val = self.attrs.get('power1_input', 0)
        if val != 0:
            val = 1
        self.attrs['psu_pwr1_all_ok'] = val
        CelRedstoneXpb_PSU2.psu_ok = val
        return cumulus.sensors.PSU_Unit.get_status(self)

class CelRedstoneXpbPsuTemp(cumulus.sensors.Temp_Unit):
    driver_hwmon = ['temp1']

    def detect_state_plat(self):
        if not self.psu.psu_ok:
            return 'ABSENT'
        return cumulus.sensors.Temp_Unit.detect_state_plat(self)

class CelRedstoneXpbTemp13(CelRedstoneXpbPsuTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp1'
    psu = CelRedstoneXpb_PSU1()
    driver_path = psu.driver_path

class CelRedstoneXpbTemp14(CelRedstoneXpbPsuTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp1'
    psu = CelRedstoneXpb_PSU2()
    driver_path = psu.driver_path

class CelRedstoneXpbSysFan(cumulus.sensors.Fan_Unit):
    driver_path = vhwmon_path + 'fan'
    no_pwm = True

class CelRedstoneXpbFan1(CelRedstoneXpbSysFan):
    name = 'Fan1'
    description = 'Fan 1 Rear'
    driver_hwmon = ['fan1']

class CelRedstoneXpbFan2(CelRedstoneXpbSysFan):
    name = 'Fan2'
    description = 'Fan 1 Front'
    driver_hwmon = ['fan2']

class CelRedstoneXpbFan3(CelRedstoneXpbSysFan):
    name = 'Fan3'
    description = 'Fan 2 Rear'
    driver_hwmon = ['fan3']

class CelRedstoneXpbFan4(CelRedstoneXpbSysFan):
    name = 'Fan4'
    description = 'Fan 2 Front'
    driver_hwmon = ['fan4']

class CelRedstoneXpbFan5(CelRedstoneXpbSysFan):
    name = 'Fan5'
    description = 'Fan 3 Rear'
    driver_hwmon = ['fan5']

class CelRedstoneXpbFan6(CelRedstoneXpbSysFan):
    name = 'Fan6'
    description = 'Fan 3 Front'
    driver_hwmon = ['fan6']

class CelRedstoneXpbFan7(CelRedstoneXpbSysFan):
    name = 'Fan7'
    description = 'Fan 4 Rear'
    driver_hwmon = ['fan7']

class CelRedstoneXpbFan8(CelRedstoneXpbSysFan):
    name = 'Fan8'
    description = 'Fan 4 Front'
    driver_hwmon = ['fan8']

class CelRedstoneXpbPsuFan(cumulus.sensors.Fan_Unit):
    driver_hwmon = ['fan1']
    no_pwm = True

    def get_status(self):
        if not self.psu.psu_ok:
            return self.update_status(None, 'not_ok')
        return cumulus.sensors.Fan_Unit.get_status(self)

    def detect_state_plat(self):
        if not self.psu.psu_ok:
            return 'ABSENT'
        return cumulus.sensors.Fan_Unit.detect_state_plat(self)

class CelRedstoneXpbFan9(CelRedstoneXpbPsuFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan1'
    psu = CelRedstoneXpb_PSU1()
    driver_path = psu.driver_path

class CelRedstoneXpbFan10(CelRedstoneXpbPsuFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan1'
    psu = CelRedstoneXpb_PSU2()
    driver_path = psu.driver_path

class CelRedstoneXpbSensors():
    units = (
        CelRedstoneXpbTemp1(),
        CelRedstoneXpbTemp2(),
        CelRedstoneXpbTemp3(),
        CelRedstoneXpbTemp4(),
        CelRedstoneXpbTemp5(),
        CelRedstoneXpbTemp6(),
        CelRedstoneXpbTemp7(),
        CelRedstoneXpbTemp8(),
        CelRedstoneXpbTemp9(),
        CelRedstoneXpbTemp10(),
        CelRedstoneXpbTemp11(),
        CelRedstoneXpbTemp12(),
        CelRedstoneXpbTemp13(),
        CelRedstoneXpbTemp14(),
        CelRedstoneXpbFan1(),
        CelRedstoneXpbFan2(),
        CelRedstoneXpbFan3(),
        CelRedstoneXpbFan4(),
        CelRedstoneXpbFan5(),
        CelRedstoneXpbFan6(),
        CelRedstoneXpbFan7(),
        CelRedstoneXpbFan8(),
        CelRedstoneXpbFan9(),
        CelRedstoneXpbFan10(),
        CelRedstoneXpb_PSU1(),
        CelRedstoneXpb_PSU2(),
    )

class CelRedstoneXpbBMC(cumulus.platform.BMC):
    name = 'bmc'
    type = 'ipmibmc'
    ignores = ['SXP_Temp_U8', 'SXP_Temp_U28', 'SXP_Temp_U1', 'SXP_Temp_U2',
               'Fan_3_Rear', 'Fan_3_Front', 'PSU2_Power_In']
    renames = { 'Fan_4_Rear': 'Fan_3_Rear', 'Fan_4_Front': 'Fan_3_Front',
                'Fan_5_Rear': 'Fan_4_Rear', 'Fan_5_Front': 'Fan_4_Front'}

class CelRedstoneXpbSwitch(CelRedstoneXpSwitch):
    def __init__(self, chip):
        CelRedstoneXpSwitch.__init__(self, chip)
        #
        # a couple of ports differ from the original Redstone XP switch
        #
        # swp49 - QSFP+_1
        #
        self.ports[48].tx_lane_map = 0x1032
        self.ports[48].tx_polarity_flip = 0

        #
        # swp54 - QSFP+_6
        #
        self.ports[53].tx_lane_map = 0x3210
        self.ports[53].tx_polarity_flip = 5


class CelRedstoneXpb(cumulus.platform.Platform):
    name = 'cel,redstone_xp_b'
    chip = cumulus.platform.TridentTwo_56854_Chip()
    switch = CelRedstoneXpbSwitch(chip)
    sensors = CelRedstoneXpbSensors()
    bmc = CelRedstoneXpbBMC()
    cpld = None
    statusleds = None
    pwms = None

Platform = CelRedstoneXpb
