#
# Copyright (c) 2019.  Cumulus Networks, Inc.  All rights reserved.
#

import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
from cumulus.platforms.cel_sea2que2 import *

class CelSeastone2MgmtSFP(cumulus.platform.TomahawkMgmt10GPort):
    def __init__(self, label, serdes, leds):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')
        cumulus.platform.TomahawkMgmt10GPort.__init__(self,
                                                      label=label,
                                                      serdes=serdes,
                                                      phy_types=None,
                                                      phy_addrs=None,
                                                      phy_layer=sfp.type.upper(),
                                                      leds=leds,
                                                      rx_lane_map=None,
                                                      rx_polarity_flip=None,
                                                      tx_lane_map=None,
                                                      tx_polarity_flip=None,
                                                      pre_emphasis=None,
                                                      oversub=True,
                                                      unit=0)
        self.sfp = sfp

class CelSeastone2QSFP28(cumulus.platform.TridentThree100GPort):

    def __init__(self, label, serdes, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.TridentThree100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   leds=None,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

# Celestica Seastone2 Switch Ports

class CelSeastone2Switch(cumulus.platform.BCMSwitch):
    _mgmtsfp = CelSeastone2MgmtSFP
    _qsfp28 = CelSeastone2QSFP28
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _qsfp28(label="1",  serdes=(  0,   1,   2,   3), tx_lane_map=0x0132, rx_lane_map=0x3210, tx_polarity_flip=0x0008, rx_polarity_flip=0x000a),
        _qsfp28(label="2",  serdes=(  4,   5,   6,   7), tx_lane_map=0x2301, rx_lane_map=0x2031, tx_polarity_flip=0x000a, rx_polarity_flip=0x000e),
        _qsfp28(label="3",  serdes=(  8,   9,  10,  11), tx_lane_map=0x0132, rx_lane_map=0x3210, tx_polarity_flip=0x0008, rx_polarity_flip=0x000a),
        _qsfp28(label="4",  serdes=( 12,  13,  14,  15), tx_lane_map=0x3201, rx_lane_map=0x2031, tx_polarity_flip=0x0002, rx_polarity_flip=0x000e),
        _qsfp28(label="5",  serdes=( 16,  17,  18,  19), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0x000a, rx_polarity_flip=0x000a),
        _qsfp28(label="6",  serdes=( 20,  21,  22,  23), tx_lane_map=0x2301, rx_lane_map=0x2031, tx_polarity_flip=0x000a, rx_polarity_flip=0x000e),
        _qsfp28(label="7",  serdes=( 24,  25,  26,  27), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0x000a, rx_polarity_flip=0x0005),
        _qsfp28(label="8",  serdes=( 28,  29,  30,  31), tx_lane_map=0x3201, rx_lane_map=0x2031, tx_polarity_flip=0x0003, rx_polarity_flip=0x0001),
        _qsfp28(label="9",  serdes=( 32,  33,  34,  35), tx_lane_map=0x0213, rx_lane_map=0x1302, tx_polarity_flip=0x0009, rx_polarity_flip=0x0001),
        _qsfp28(label="10", serdes=( 36,  37,  38,  39), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x0007, rx_polarity_flip=0x0009),
        _qsfp28(label="11", serdes=( 40,  41,  42,  43), tx_lane_map=0x0231, rx_lane_map=0x3120, tx_polarity_flip=0x000d, rx_polarity_flip=0x000b),
        _qsfp28(label="12", serdes=( 44,  45,  46,  47), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x0007, rx_polarity_flip=0x000c),
        _qsfp28(label="13", serdes=( 48,  49,  50,  51), tx_lane_map=0x2103, rx_lane_map=0x3120, tx_polarity_flip=0x000b, rx_polarity_flip=0x000c),
        _qsfp28(label="14", serdes=( 52,  53,  54,  55), tx_lane_map=0x2301, rx_lane_map=0x2031, tx_polarity_flip=0x000a, rx_polarity_flip=0x000e),
        _qsfp28(label="15", serdes=( 56,  57,  58,  59), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=0x000b, rx_polarity_flip=0x000a),
        _qsfp28(label="16", serdes=( 60,  61,  62,  63), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=0x0002, rx_polarity_flip=0x0005),
        _qsfp28(label="17", serdes=( 64,  65,  66,  67), tx_lane_map=0x3210, rx_lane_map=0x1023, tx_polarity_flip=0x0005, rx_polarity_flip=0x0004),
        _qsfp28(label="18", serdes=( 68,  69,  70,  71), tx_lane_map=0x0123, rx_lane_map=0x1302, tx_polarity_flip=0x0005, rx_polarity_flip=0x0001),
        _qsfp28(label="19", serdes=( 72,  73,  74,  75), tx_lane_map=0x2301, rx_lane_map=0x1032, tx_polarity_flip=0x000a, rx_polarity_flip=0x0005),
        _qsfp28(label="20", serdes=( 76,  77,  78,  79), tx_lane_map=0x2013, rx_lane_map=0x3120, tx_polarity_flip=0x000c, rx_polarity_flip=0x000c),
        _qsfp28(label="21", serdes=( 80,  81,  82,  83), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x000c, rx_polarity_flip=0x0004),
        _qsfp28(label="22", serdes=( 84,  85,  86,  87), tx_lane_map=0x0123, rx_lane_map=0x2130, tx_polarity_flip=0x0005, rx_polarity_flip=0x0003),
        _qsfp28(label="23", serdes=( 88,  89,  90,  91), tx_lane_map=0x2301, rx_lane_map=0x2031, tx_polarity_flip=0x0005, rx_polarity_flip=0x000c),
        _qsfp28(label="24", serdes=( 92,  93,  94,  95), tx_lane_map=0x0312, rx_lane_map=0x2310, tx_polarity_flip=0x0003, rx_polarity_flip=0x000b),
        _qsfp28(label="25", serdes=( 96,  97,  98,  99), tx_lane_map=0x2301, rx_lane_map=0x1032, tx_polarity_flip=0x0005, rx_polarity_flip=0x0005),
        _qsfp28(label="26", serdes=(100, 101, 102, 103), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0x0005, rx_polarity_flip=0x0002),
        _qsfp28(label="27", serdes=(104, 105, 106, 107), tx_lane_map=0x2301, rx_lane_map=0x1032, tx_polarity_flip=0x0005, rx_polarity_flip=0x000a),
        _qsfp28(label="28", serdes=(108, 109, 110, 111), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0x0005, rx_polarity_flip=0x0005),
        _qsfp28(label="29", serdes=(112, 113, 114, 115), tx_lane_map=0x2301, rx_lane_map=0x2031, tx_polarity_flip=0x0005, rx_polarity_flip=0x0001),
        _qsfp28(label="30", serdes=(116, 117, 118, 119), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0x0005, rx_polarity_flip=0x0005),
        _qsfp28(label="31", serdes=(120, 121, 122, 123), tx_lane_map=0x2301, rx_lane_map=0x1032, tx_polarity_flip=0x0005, rx_polarity_flip=0x000a),
        _qsfp28(label="32", serdes=(124, 125, 126, 127), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0x0005, rx_polarity_flip=0x0005),

#        _mgmtsfp(label="33", serdes=(130,), leds=(2,126)),
    )

# Intel Denverton CPU

#class CelSea2Que2CoreTemp(cumulus.sensors.Temp_Unit):
#    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
#    driver_path = '/sys/devices/platform/coretemp.0'
#    no_shutdown = True
#    temp_crit = 95
#    temp_max = 75
#
#class CelSea2Que2CPUTemp1(CelSea2Que2CoreTemp):
#    name = 'Temp22'
#    description = 'Physical id 0'
#    driver_hwmon = ['temp1']
#
#class CelSea2Que2CPUTemp2(CelSea2Que2CoreTemp):
#    name = 'Temp23'
#    description = 'Core 6'
#    driver_hwmon = ['temp8']
#
#class CelSea2Que2CPUTemp3(CelSea2Que2CoreTemp):
#    name = 'Temp24'
#    description = 'Core 12'
#    driver_hwmon = ['temp14']

# Sensors class

class CelSeastone2Sensors():
    units = (CelSea2Que2ASICTemp1(),
             CelSea2Que2ASICTemp2(),
             CelSea2Que2ASICTemp3(),
             CelSea2Que2ASICTemp4(),
             CelSea2Que2ASICTemp5(),
             CelSea2Que2ASICTemp6(),
             CelSea2Que2ASICTemp7(),
             CelSea2Que2ASICTemp8(),
             CelSea2Que2ASICTemp9(),
             CelSea2Que2ASICTemp10(),
             CelSea2Que2ASICTemp11(),
             CelSea2Que2ASICTemp12(),
             CelSea2Que2BMCTemp1(),
             CelSea2Que2BMCTemp2(),
             CelSea2Que2BMCTemp3(),
             CelSea2Que2BMCTemp4(),
             CelSea2Que2BMCTemp5(),
             CelSea2Que2BMCTemp6(),
             CelSea2Que2BMCTemp7(),
             CelSea2Que2BMCTemp8(),
             CelSea2Que2BMCTemp9(),
             CelSea2Que2Fan1(),
             CelSea2Que2Fan2(),
             CelSea2Que2Fan3(),
             CelSea2Que2Fan4(),
             CelSea2Que2Fan5(),
             CelSea2Que2Fan6(),
             CelSea2Que2Fan7(),
             CelSea2Que2Fan8(),
             CelSea2Que2PSU1(),
             CelSea2Que2PSU2(),
             CelSea2Que2PSU1Temp1(),
             CelSea2Que2PSU1Temp2(),
             CelSea2Que2PSU1Fan1(),
             CelSea2Que2PSU2Temp1(),
             CelSea2Que2PSU2Temp2(),
             CelSea2Que2PSU2Fan1(),
             )

class CelSeastone2(cumulus.platform.Platform):
    name = 'cel,seastone_2'
    chip = cumulus.platform.TridentThreeX7_56870_Chip()
    switch = CelSeastone2Switch(chip)
    bmc = CelSea2Que2BMC()
    sensors = CelSeastone2Sensors()
    statusleds = None
    pwms = None

Platform = CelSeastone2
