#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

''' DELL s3000
'''
class DellS3000SFPp(cumulus.platform.Helix410GPort):
    _set_led = '''
;;; DellS3000SFPp
;;;
;;; Two bits per port:
;;;
;;;                 1st bit
;;;                0        1
;;;            +-------+-------+
;;;          0 |   on  |  off  |
;;;  2nd bit   +-------+-------+
;;;          1 | blink |  off  |
;;;            +-------+-------+
;;;
set_led:
        port a


        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack                    ; push 1st bit (active low)

        pushst  PORTSTATUS_ZERO
        pack                    ; push second bit (active low)

        ret

'''
    def __init__(self, label, hw_port, leds, rx_polarity_flip = None, tx_polarity_flip=None,
                        tx_lane_map=None, rx_lane_map=None):
        cumulus.platform.Helix410GPort.__init__(self,
                                                 label=label,
                                                 hw_port=hw_port,
                                                 phy_type=None,
                                                 phy_addr=None,
                                                 phy_layer='SFP+',
                                                 leds=leds,
                                                 rx_lane_map=rx_lane_map,
                                                 tx_lane_map=tx_lane_map,
                                                 rx_polarity_flip=rx_polarity_flip,
                                                 tx_polarity_flip=tx_polarity_flip,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        '''
        self._check_mode(mode)
        '''

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, 0)

        call = (fn_name, self._set_led, led_port)

        led_index = led_index_list[0]
        set_calls[led_index] = call
        set_calls[led_index + 1] = None

        return set_calls

class DellS3000GigE(cumulus.platform.Helix41GPort):
    def __init__(self, label, hw_port, phy_addr, prim_offset):
        cumulus.platform.Helix41GPort.__init__(self,
                                                label=label,
                                                hw_port=hw_port,
                                                phy_type="54282",
                                                phy_addr=phy_addr,
                                                phy_layer='1GBaseT',
                                                prim_offset=prim_offset,
                                                leds=None,
                                                unit=0)

class DellS3000LED(cumulus.ledup.Helix4LEDController):
    num_leds = 8

class DellS3000Switch(cumulus.platform.BCMSwitch):
    '''
    led chain starting from bit 1 and ending on bit 40....
     1 : sfp_45_act, sfp_45_link_g, sfp_45_link_y,
     4 : sfp_46_act, sfp_46_link_g, sfp_46_link_y,
     7 : sfp_47_act, sfp_47_link_g, sfp_47_link_y,
    10 : sfp_48_act, sfp_48_link_g, sfp_48_link_y,
    13 : rj_45_act, 1, rj_45_link_g, rj_45_link_y,
    17 : rj_46_act, 1, rj_46_link_g, rj_46_link_y,
    21 : rj_47_act, 1, rj_47_link_g, rj_47_link_y,
    25 : rj_48_act, 1, rj_48_link_g, rj_48_link_y,
    29 : mod_a_0, mod_a_1, mod_a_2,
    32 : mod_b_0, mod_b_1, mod_b_2,
    35 : 1, 1, 1,
    38 : 1, 1, 1
    '''
    ledup = DellS3000LED
    _gige = DellS3000GigE
    _sfp = DellS3000SFPp
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = True

    ports = (
        _gige(label="1",  hw_port='ge1',  phy_addr=0x2,  prim_offset=0x0001),
        _gige(label="2",  hw_port='ge0',  phy_addr=0x1,  prim_offset=0x0000),
        _gige(label="3",  hw_port='ge3',  phy_addr=0x4,  prim_offset=0x0003),
        _gige(label="4",  hw_port='ge2',  phy_addr=0x3,  prim_offset=0x0002),
        _gige(label="5",  hw_port='ge5',  phy_addr=0x6,  prim_offset=0x0005),
        _gige(label="6",  hw_port='ge4',  phy_addr=0x5,  prim_offset=0x0004),
        _gige(label="7",  hw_port='ge7',  phy_addr=0x8,  prim_offset=0x0007),
        _gige(label="8",  hw_port='ge6',  phy_addr=0x7,  prim_offset=0x0006),
        _gige(label="9",  hw_port='ge9',  phy_addr=0xb,  prim_offset=0x0801),
        _gige(label="10", hw_port='ge8' , phy_addr=0xa,  prim_offset=0x0800),
        _gige(label="11", hw_port='ge11', phy_addr=0xd,  prim_offset=0x0803),
        _gige(label="12", hw_port='ge10', phy_addr=0xc,  prim_offset=0x0802),
        _gige(label="13", hw_port='ge13', phy_addr=0xf,  prim_offset=0x0805),
        _gige(label="14", hw_port='ge12', phy_addr=0xe,  prim_offset=0x0804),
        _gige(label="15", hw_port='ge15', phy_addr=0x11, prim_offset=0x0807),
        _gige(label="16", hw_port='ge14', phy_addr=0x10, prim_offset=0x0806),
        _gige(label="17", hw_port='ge17', phy_addr=0x14, prim_offset=0x1001),
        _gige(label="18", hw_port='ge16', phy_addr=0x13, prim_offset=0x1000),
        _gige(label="19", hw_port='ge19', phy_addr=0x16, prim_offset=0x1003),
        _gige(label="20", hw_port='ge18', phy_addr=0x15, prim_offset=0x1002),
        _gige(label="21", hw_port='ge21', phy_addr=0x18, prim_offset=0x1005),
        _gige(label="22", hw_port='ge20', phy_addr=0x17, prim_offset=0x1004),
        _gige(label="23", hw_port='ge23', phy_addr=0x1a, prim_offset=0x1007),
        _gige(label="24", hw_port='ge22', phy_addr=0x19, prim_offset=0x1006),
        _gige(label="25", hw_port='ge25', phy_addr=0x22, prim_offset=0x1801),
        _gige(label="26", hw_port='ge24', phy_addr=0x21, prim_offset=0x1800),
        _gige(label="27", hw_port='ge27', phy_addr=0x24, prim_offset=0x1803),
        _gige(label="28", hw_port='ge26', phy_addr=0x23, prim_offset=0x1802),
        _gige(label="29", hw_port='ge29', phy_addr=0x26, prim_offset=0x1805),
        _gige(label="30", hw_port='ge28', phy_addr=0x25, prim_offset=0x1804),
        _gige(label="31", hw_port='ge31', phy_addr=0x28, prim_offset=0x1807),
        _gige(label="32", hw_port='ge30', phy_addr=0x27, prim_offset=0x1806),
        _gige(label="33", hw_port='ge33', phy_addr=0x2b, prim_offset=0x2001),
        _gige(label="34", hw_port='ge32', phy_addr=0x2a, prim_offset=0x2000),
        _gige(label="35", hw_port='ge35', phy_addr=0x2d, prim_offset=0x2003),
        _gige(label="36", hw_port='ge34', phy_addr=0x2c, prim_offset=0x2002),
        _gige(label="37", hw_port='ge37', phy_addr=0x2f, prim_offset=0x2005),
        _gige(label="38", hw_port='ge36', phy_addr=0x2e, prim_offset=0x2004),
        _gige(label="39", hw_port='ge39', phy_addr=0x31, prim_offset=0x2007),
        _gige(label="40", hw_port='ge38', phy_addr=0x30, prim_offset=0x2006),
        _gige(label="41", hw_port='ge41', phy_addr=0x34, prim_offset=0x2801),
        _gige(label="42", hw_port='ge40', phy_addr=0x33, prim_offset=0x2800),
        _gige(label="43", hw_port='ge43', phy_addr=0x36, prim_offset=0x2803),
        _gige(label="44", hw_port='ge42', phy_addr=0x35, prim_offset=0x2802),
        _gige(label="45", hw_port='ge45', phy_addr=0x38, prim_offset=0x2805),
        _gige(label="46", hw_port='ge44', phy_addr=0x37, prim_offset=0x2804),
        _gige(label="47", hw_port='ge47', phy_addr=0x3a, prim_offset=0x2807),
        _gige(label="48", hw_port='ge46', phy_addr=0x39, prim_offset=0x2806),
        _sfp(label="49",  hw_port='hg1',  leds=(0,2)),
        _sfp(label="50",  hw_port='hg0',  leds=(0,0),
                tx_lane_map=0x2301, rx_lane_map=0x1032,
                rx_polarity_flip=True, tx_polarity_flip=True),
        _sfp(label="51",  hw_port='hg3',  leds=(0,6)),
        _sfp(label="52",  hw_port='hg2',  leds=(0,4),
                rx_polarity_flip=True, tx_polarity_flip=True),
    )

DellS3000_i2c_path = '/sys/bus/i2c/devices/'

''' max6699 temp sensor 1 '''
DellS3000_i2c_11_1a_path = DellS3000_i2c_path + '11-001a'

''' max6699 temp sensor 2 '''
DellS3000_i2c_21_1a_path = DellS3000_i2c_path + '21-001a'

DellS3000_i2c_10_4d_path = DellS3000_i2c_path + '10-004d'

''' emc2305: Fan Tray 1, 2, 3 Controller '''
DellS3000_i2c_35_4d_path = DellS3000_i2c_path + '35-004d'

''' dps200: PSU1 Temp/Fan '''
DellS3000_i2c_23_5b_path = DellS3000_i2c_path + '23-005b'

''' dps200: PSU2 Temp/Fan '''
DellS3000_i2c_22_5a_path = DellS3000_i2c_path + '22-005a'

DellS3000_cpld_path =  '/sys/bus/platform/devices/dell_s3000_cpld.0'

class DellS3000CPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/dell_s3000_cpld.0'

'''
   Dell S3000 LEDS
     System:
       green - normal operation
       amber blinking - Fault
     Master:
       off - switch is in slave mode (normal operation)
       green - switch is in stacking mode
     Power:
       off - no power
       amber blinking - Power Fault
       green - normal operation
     Fan:
       green - fan powered and running at expected rpm
       amber - fan failed (including incompatible airflow direction)
     Fan_Tray_X (1 - 3):
       green - fan X installed and working
       yellow - fan X malfunction
'''

class DellS3000SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_system_status'
    bad_led_color = 'amber_blinking'
    led_path =  DellS3000_cpld_path

class DellS3000PowerLED(cumulus.sensors.StatusLED_Unit):
    name = 'Power'
    sources = ['PSU1', 'PSU2']
    led_location = 'led_system_power'
    bad_led_color = 'amber_blinking'
    led_path =   DellS3000_cpld_path

class DellS3000MasterLED(cumulus.sensors.StatusLED_Unit):
    name = 'Master'
    led_location = 'led_system_master'
    bad_led_color = 'off'
    good_led_color = 'off'
    led_path =   DellS3000_cpld_path

class DellS3000FanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fans'
    sources = ['Fan1', 'Fan2', 'Fan3']
    led_location = 'led_system_fan'
    bad_led_color = 'amber'
    led_path =   DellS3000_cpld_path

class DellS3000Fan1LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 1'
    sources = ['Fan1']
    led_location = 'led_fan1'
    bad_led_color = 'yellow_blinking'
    led_path =   DellS3000_cpld_path

class DellS3000Fan2LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 2'
    sources = ['Fan2']
    led_location = 'led_fan2'
    bad_led_color = 'yellow_blinking'
    led_path =   DellS3000_cpld_path

class DellS3000Fan3LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 3'
    sources = ['Fan3']
    led_location = 'led_fan3'
    bad_led_color = 'yellow_blinking'
    led_path =   DellS3000_cpld_path

class DellS3000CoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_fan_min = 65
    temp_max = 90
    temp_crit = 95

class DellS3000InletTemp(cumulus.sensors.Temp_Unit):
    temp_crit = 58
    temp_max = 49
    temp_fan_min = 31

class DellS3000MacTemp(cumulus.sensors.Temp_Unit):
    temp_crit = 90
    temp_max = 65
    temp_fan_min = 50

class DellS3000Temp1(DellS3000CoreTemp):
    name = 'Temp1'
    description = 'Core 0'
    driver_hwmon = ['temp2']

class DellS3000Temp2(DellS3000CoreTemp):
    name = 'Temp2'
    description = 'Core 1'
    driver_hwmon = ['temp3']

class DellS3000Temp3(DellS3000CoreTemp):
    name = 'Temp3'
    description = 'CPU sensor F2R'
    driver_path = DellS3000_i2c_11_1a_path
    driver_hwmon = ['temp2']

class DellS3000Temp4(DellS3000CoreTemp):
    name = 'Temp4'
    description = 'CPU sensor R2F'
    driver_path = DellS3000_i2c_11_1a_path
    driver_hwmon = ['temp3']

class DellS3000Temp5(DellS3000InletTemp):
    name = 'Temp5'
    description = 'Inlet ambient sensor'
    driver_path = DellS3000_i2c_21_1a_path
    driver_hwmon = ['temp1']

class DellS3000Temp6(DellS3000MacTemp):
    name = 'Temp6'
    description =  cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = DellS3000_i2c_21_1a_path
    driver_hwmon = ['temp2']

class DellS3000Temp7(DellS3000InletTemp):
    name = 'Temp7'
    description = 'Right Inlet ambient sensor'
    driver_path = DellS3000_i2c_21_1a_path
    driver_hwmon = ['temp3']

class DellS3000Temp8(DellS3000MacTemp):
    name = 'Temp8'
    description =  cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = DellS3000_i2c_21_1a_path
    driver_hwmon = ['temp4']

class DellS3000Temp9(DellS3000InletTemp):
    name = 'Temp9'
    description = 'Left Inlet ambient sensor'
    driver_path = DellS3000_i2c_21_1a_path
    driver_hwmon = ['temp5']

class DellS3000Fan(cumulus.sensors.Fan_Unit):
    fan_max = 19000
    fan_min = 8200
    minpwm = 102
    maxpwm = 255
    hwmon_search = False

class DellS3000Fan1(DellS3000Fan):
    name = 'Fan1'
    description = 'Fan Tray 1'
    driver_path = DellS3000_i2c_35_4d_path
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']
    cpld_path =   DellS3000_cpld_path
    cpld_hwmon = ['fan_1']

class DellS3000Fan2(DellS3000Fan):
    name = 'Fan2'
    description = 'Fan Tray 2'
    driver_path = DellS3000_i2c_35_4d_path
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']
    cpld_path =   DellS3000_cpld_path
    cpld_hwmon = ['fan_2']

class DellS3000Fan3(DellS3000Fan):
    name = 'Fan3'
    description = 'Fan Tray 3'
    driver_path = DellS3000_i2c_35_4d_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    cpld_path =   DellS3000_cpld_path
    cpld_hwmon = ['fan_3']

class DellS3000PSU(cumulus.sensors.PSU_Unit):

    def find_and_add_attrs(self):
        attr_dict = { ''       : '_present',
                      '_ac'    : '_ac_ok',
                      '_power' : '_dc_ok',
                    }
        err = 0
        ok_cnt = 0
        for p, n in attr_dict.iteritems():
            fname = os.path.join(self.cpld_path, self.cpld_hwmon[0] + p)
            with open(fname) as fp:
                try:
                    val = fp.read().rsplit('\n')[0]
                    # In case of failure, it would be not_installed or power_bad.
                    # In case of good, it will be installed or ok.
                    if (val == 'installed') or (val == 'ok'):
                        self.attrs[self.cpld_hwmon[0] + n] = 1
                        ok_cnt += 1
                    else:
                        self.attrs[self.cpld_hwmon[0] + n] = 0
                except:
                    self.update_msg('Unable to read device')
                    err |= -1
        if ok_cnt == len(attr_dict):
            self.attrs[self.cpld_hwmon[0] + '_all_ok'] = 1
        else:
            self.attrs[self.cpld_hwmon[0] + '_all_ok'] = 0
        return err

class DellS3000PSU1(DellS3000PSU):
    name = 'PSU1'
    cpld_path =   DellS3000_cpld_path
    cpld_hwmon = ['psu_pwr1']

class DellS3000PSU2(DellS3000PSU):
    name = 'PSU2'
    cpld_path =   DellS3000_cpld_path
    cpld_hwmon = ['psu_pwr2']

class DellS3000PSUTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 30
    temp_max = 85
    temp_crit = 90

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

class DellS3000PSU1Temp1(DellS3000PSUTemp):
    name = 'PSU1Temp1'
    driver_path = DellS3000_i2c_23_5b_path 
    driver_hwmon = ['temp1']
    description = 'PSU1 Temp Sensor'        
    psu = 'PSU1'

class DellS3000PSU1Temp2(DellS3000PSUTemp):
    name = 'PSU1Temp2'
    driver_path =DellS3000_i2c_23_5b_path 
    driver_hwmon = ['temp2']
    description = 'PSU1 Temp Sensor'        
    psu = 'PSU1'

class DellS3000PSU2Temp1(DellS3000PSUTemp):
    name = 'PSU2Temp1'
    driver_path =DellS3000_i2c_22_5a_path
    driver_hwmon = ['temp1']
    description = 'PSU2 Temp Sensor'        
    psu = 'PSU2'

class DellS3000PSU2Temp2(DellS3000PSUTemp):
    name = 'PSU2Temp2'
    driver_path =DellS3000_i2c_22_5a_path
    driver_hwmon = ['temp2']
    description = 'PSU2 Temp Sensor'
    psu = 'PSU2'

class DellS3000PSUFan(cumulus.sensors.Fan_Unit):
    minpwm = 64
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']        

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class DellS3000PSU1Fan(DellS3000PSUFan):
    name = 'PSU1Fan'
    driver_path = DellS3000_i2c_23_5b_path 
    description = 'PSU1 Fan'
    psu = 'PSU1'

class DellS3000PSU2Fan(DellS3000PSUFan):
    name = 'PSU2Fan'
    driver_path =DellS3000_i2c_22_5a_path
    description = 'PSU2 Fan'
    psu = 'PSU2'

class DellS3000PWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (DellS3000Fan1(),
            DellS3000Fan2(),
            DellS3000Fan3(),
            DellS3000PSU1Fan(),
            DellS3000PSU2Fan(),
            )

    temps = (DellS3000Temp1(),
             DellS3000Temp2(),
             DellS3000Temp3(),
             DellS3000Temp4(),
             DellS3000Temp5(),
             DellS3000Temp6(),
             DellS3000Temp7(),
             DellS3000Temp8(),
             DellS3000Temp9(),
             DellS3000PSU1Temp1(),
             DellS3000PSU1Temp2(),
             DellS3000PSU2Temp1(),
             DellS3000PSU2Temp2(),
             )

class DellS3000Sensors():
    units = (DellS3000PSU1(),
             DellS3000PSU2(),
             DellS3000Temp1(),
             DellS3000Temp2(),
             DellS3000Temp3(),
             DellS3000Temp4(),
             DellS3000Temp5(),
             DellS3000Temp6(),
             DellS3000Temp7(),
             DellS3000Temp8(),
             DellS3000Temp9(),
             DellS3000Fan1(),
             DellS3000Fan2(),
             DellS3000Fan3(),
             DellS3000PSU1Fan(),
             DellS3000PSU2Fan(),
             DellS3000PSU1Temp1(),
             DellS3000PSU1Temp2(),
             DellS3000PSU2Temp1(),
             DellS3000PSU2Temp2(),
             )

class DellS3000PWMs():
    units = (DellS3000PWM1(),
             )

class DellS3000LEDs():
    units = (
             DellS3000SystemLED(),
             DellS3000PowerLED(),
             DellS3000MasterLED(),
             DellS3000FanLED(),
             DellS3000Fan1LED(),
             DellS3000Fan2LED(),
             DellS3000Fan3LED(),
        )

class DellS3000(cumulus.platform.Platform):
    name = 'dell,s3000_c2338'
    chip = cumulus.platform.Helix4Chip()
    switch = DellS3000Switch(chip)
    statusleds = DellS3000LEDs()
    pwms = DellS3000PWMs()
    sensors = DellS3000Sensors()

Platform = DellS3000
