#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class DellS6010CPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/dell_s6010_cpld.0'

DellS6010_i2c_path = '/sys/bus/i2c/devices/'

''' tmp75: temp sensor 1 '''
DellS6010_i2c_11_4a_path = DellS6010_i2c_path + '11-004a'

''' tmp75: temp sensor 2 '''
DellS6010_i2c_11_4b_path = DellS6010_i2c_path + '11-004b'

''' tmp75: temp sensor 3 '''
DellS6010_i2c_11_4c_path = DellS6010_i2c_path + '11-004c'

''' tmp75: temp sensor 4 '''
DellS6010_i2c_11_4e_path = DellS6010_i2c_path + '11-004e'

''' tmp75: temp sensor 5 '''
DellS6010_i2c_11_4f_path = DellS6010_i2c_path + '11-004f'

''' tmp75: temp sensor 6 '''
DellS6010_i2c_10_4d_path = DellS6010_i2c_path + '10-004d'

''' dps460: PSU1 Temp/Fan '''
DellS6010_i2c_14_58_path = DellS6010_i2c_path + '14-0058'

''' dps460: PSU2 Temp/Fan '''
DellS6010_i2c_14_59_path = DellS6010_i2c_path + '14-0059'

''' emc2305: Fan Controller '''
DellS6010_i2c_11_4d_path = DellS6010_i2c_path + '11-004d'

''' ltc4215: PSU1 '''
DellS6010_i2c_11_40_path = DellS6010_i2c_path + '11-0040'

''' ltc4215: PSU2 '''
DellS6010_i2c_11_42_path = DellS6010_i2c_path + '11-0042'

class DellS6010SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_system'
    bad_led_color = 'yellow_blinking'

class DellS6010PowerLED(cumulus.sensors.StatusLED_Unit):
    name = 'Power'
    sources = ['PSU1', 'PSU2']
    led_location = 'led_power'
    bad_led_color = 'yellow_blinking'

class DellS6010FanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan'
    sources = ['Fan1', 'Fan2','Fan3', 'Fan4','Fan5']
    led_location = 'led_fan'
    bad_led_color = 'yellow_blinking'
    
class DellS6010Fan1LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 1'
    sources = ['Fan1']
    led_location = 'led_fan_tray_1'
    bad_led_color = 'yellow'

class DellS6010Fan2LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 2'
    sources = ['Fan2']
    led_location = 'led_fan_tray_2'
    bad_led_color = 'yellow'

class DellS6010Fan3LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 3'
    sources = ['Fan3']
    led_location = 'led_fan_tray_3'
    bad_led_color = 'yellow'

class DellS6010Fan4LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 4'
    sources = ['Fan4']
    led_location = 'led_fan_tray_4'
    bad_led_color = 'yellow'

class DellS6010Fan5LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 5'
    sources = ['Fan5']
    led_location = 'led_fan_tray_5'
    bad_led_color = 'yellow'
    
class DellS6010Temp1(cumulus.sensors.Temp_Unit):
    # U269, Hardware shutdown, Fan Controlled
    name = 'Temp1'
    description = 'Board Sensor near Front Left Vent'
    driver_path = DellS6010_i2c_11_4b_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True
    
class DellS6010Temp1F2B(DellS6010Temp1):
    temp_fan_min = 28
    temp_fan_max = 56    
    temp_max = 58
    temp_crit = 62

class DellS6010Temp1B2F(DellS6010Temp1):
    temp_fan_min = 32
    temp_fan_max = 54
    temp_max = 56
    temp_crit = 60

class DellS6010Temp2(cumulus.sensors.Temp_Unit):
    # U270, Hardware shutdown, Fan Controlled
    name = 'Temp2'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = DellS6010_i2c_11_4a_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True
            
class DellS6010Temp2F2B(DellS6010Temp2):
    temp_fan_min = 30
    temp_fan_max = 58
    temp_max = 65
    temp_crit = 72
        
class DellS6010Temp2B2F(DellS6010Temp2):
    temp_fan_min = 34
    temp_fan_max = 60    
    temp_max = 62
    temp_crit = 79
        
class DellS6010Temp3(cumulus.sensors.Temp_Unit):
    # U54, Hardware shutdown, Fan Controlled
    name = 'Temp3'
    description = 'Board Sensor near Front Right Vent'
    driver_path = DellS6010_i2c_11_4c_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True
        
class DellS6010Temp3F2B(DellS6010Temp3):
    temp_fan_min = 28
    temp_fan_max = 54
    temp_max = 56
    temp_crit = 60
    
class DellS6010Temp3B2F(DellS6010Temp3):
    temp_fan_min = 32
    temp_fan_max = 57
    temp_max = 59
    temp_crit = 63
            
class DellS6010Temp4(cumulus.sensors.Temp_Unit):
    # U245, Hardware shutdown, Fan Controlled
    name = 'Temp4'
    description = 'Board Sensor near QSFP'
    driver_path = DellS6010_i2c_11_4e_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True
        
class DellS6010Temp4F2B(DellS6010Temp4):
    temp_fan_min = 28
    temp_fan_max = 56
    temp_max = 61
    temp_crit = 66

class DellS6010Temp4B2F(DellS6010Temp4):
    temp_fan_min = 32
    temp_fan_max = 58
    temp_max = 66
    temp_crit = 71

class DellS6010Temp5(cumulus.sensors.Temp_Unit):
    # U334, Hardware shutdown, Fan Controlled
    name = 'Temp5'
    description = 'Board Sensor near Fan'
    driver_path = DellS6010_i2c_11_4f_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True
        
class DellS6010Temp5F2B(DellS6010Temp5):
    temp_fan_min = 32
    temp_fan_max = 54    
    temp_max = 59
    temp_crit = 64

class DellS6010Temp5B2F(DellS6010Temp5):
    temp_fan_min = 34
    temp_fan_max = 56    
    temp_max = 59
    temp_crit = 64
    
class DellS6010Temp6(cumulus.sensors.Temp_Unit):
    # U57, Hardware shutdown, Fan Controlled
    name = 'Temp6'
    description = 'Board Sensor near CPU'
    driver_path = DellS6010_i2c_10_4d_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True
        
class DellS6010Temp6F2B(DellS6010Temp6):
    temp_fan_min = 28
    temp_fan_max = 56    
    temp_max = 61
    temp_crit = 66

class DellS6010Temp6B2F(DellS6010Temp6):
    temp_fan_min = 32
    temp_fan_max = 58    
    temp_max = 60
    temp_crit = 65

class DellS6010CT(cumulus.sensors.Temp_Unit):
    # U16
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_fan_min = 75
    temp_fan_max = 90    
    temp_max = 104
    temp_crit = 125
    
class DellS6010Temp7(DellS6010CT):
    name = 'Temp7'
    driver_hwmon = ['temp2']

class DellS6010Temp8(DellS6010CT):
    name = 'Temp8'
    driver_hwmon = ['temp3']
        
class DellS6010PSUTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 30
    temp_fan_max = 52
    temp_max = 55
    temp_crit = 65
    psu = None
    temp_otp_hw = True

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state == 'ABSENT'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

class DellS6010Temp9(DellS6010PSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp Sensor'
    driver_path = DellS6010_i2c_14_58_path
    driver_hwmon = ['temp1']
    psu = 'PSU1'
        
class DellS6010Temp10(DellS6010PSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp Sensor'
    driver_path = DellS6010_i2c_14_59_path
    driver_hwmon = ['temp1']
    psu = 'PSU2'
            
class DellS6010T2PlusTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 105
    temp_crit    = 110
    temp_fan_min = 40
    temp_fan_max = 68

class DellS6010T2PlusTemp1(DellS6010T2PlusTemp):
    name = 'Temp9'
    driver_hwmon = ['temp1']

class DellS6010T2PlusTemp2(DellS6010T2PlusTemp):
    name = 'Temp10'
    driver_hwmon = ['temp2']

class DellS6010T2PlusTemp3(DellS6010T2PlusTemp):
    name = 'Temp11'
    driver_hwmon = ['temp3']

class DellS6010T2PlusTemp4(DellS6010T2PlusTemp):
    name = 'Temp12'
    driver_hwmon = ['temp4']

class DellS6010T2PlusTemp5(DellS6010T2PlusTemp):
    name = 'Temp13'
    driver_hwmon = ['temp5']

class DellS6010T2PlusTemp6(DellS6010T2PlusTemp):
    name = 'Temp14'
    driver_hwmon = ['temp6']

class DellS6010T2PlusTemp7(DellS6010T2PlusTemp):
    name = 'Temp15'
    driver_hwmon = ['temp7']

class DellS6010T2PlusTemp8(DellS6010T2PlusTemp):
    name = 'Temp16'
    driver_hwmon = ['temp8']

class DellS6010T2PlusTemp9(DellS6010T2PlusTemp):
    name = 'Temp17'
    driver_hwmon = ['temp9']

class DellS6010Fan(cumulus.sensors.Fan_Unit):
    fan_max = 23000 
    fan_min = 11000
    minpwm = 128
    hwmon_search = False

class DellS6010PSUFan(cumulus.sensors.Fan_Unit):
    minpwm = 128 # Corresponds to duty 50 % and RPM 11500
    fan_max = 19000
    fan_min = 8000
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state == 'ABSENT'):
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class DellS6010Fan1(DellS6010Fan):
    name = 'Fan1'
    description = 'Fan Tray 1, Fan 1'
    driver_path = DellS6010_i2c_11_4d_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    cpld_hwmon = ['fan_1']
    
class DellS6010Fan2(DellS6010Fan):
    name = 'Fan2'
    description = 'Fan Tray 2, Fan 1'
    driver_path = DellS6010_i2c_11_4d_path
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']
    cpld_hwmon = ['fan_2']
    
class DellS6010Fan3(DellS6010Fan):
    name = 'Fan3'
    description = 'Fan Tray 3, Fan 1'
    driver_path = DellS6010_i2c_11_4d_path
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']    
    cpld_hwmon = ['fan_3']
    
class DellS6010Fan4(DellS6010Fan):
    name = 'Fan4'
    description = 'Fan Tray 4, Fan 1'
    driver_path = DellS6010_i2c_11_4d_path
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']
    cpld_hwmon = ['fan_4']
    
class DellS6010Fan5(DellS6010Fan):
    name = 'Fan5'
    description = 'Fan Tray 1, Fan 2'
    driver_path = DellS6010_i2c_11_4d_path
    driver_hwmon = ['fan5']
    pwm_hwmon = ['pwm5']
    cpld_hwmon = ['fan_5']
    
class DellS6010Fan6(DellS6010PSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    driver_path = DellS6010_i2c_14_58_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    psu = 'PSU1'
        
class DellS6010Fan7(DellS6010PSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    driver_path = DellS6010_i2c_14_59_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']    
    psu = 'PSU2'
    
class DellS6010PSU(cumulus.sensors.PSU_Unit):
    init_done = False
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            subprocess.call(["/sbin/rmmod", "dps460"])
            subprocess.call(["/sbin/modprobe", "dps460"])
            self.init_done = True

class DellS6010PSU1(DellS6010PSU):
    name = 'PSU1'
    #driver_path = DellS6010_i2c_11_42_path
    #driver_hwmon = ['power1', 'in1', 'in2', 'curr1']
    cpld_hwmon = ['psu_pwr1']
    
class DellS6010PSU2(DellS6010PSU):
    name = 'PSU2'
    #driver_path = DellS6010_i2c_11_40_path
    #driver_hwmon = ['power1', 'in1', 'in2', 'curr1']
    cpld_hwmon = ['psu_pwr2']
    
def dell_s6010_airflow():
    f2b_part_nums = ['083R0P', '01YRKK', '0N53X1', '036DN0']
    b2f_part_nums = ['0FHD1H', '03H3JY', '0Y9TW5', '0K41TC']

    try:
        plat = subprocess.check_output(('platform-detect', '-m'))
    except OSError:
        return None
    if plat.strip() != 'dell,s6010_c2538':
        return None

    try:
       b_info = subprocess.check_output('/usr/cumulus/bin/decode-syseeprom')
    except:
       b_info = ''
    # Part Number Format
    # TLV Name             Code Len Value
    # -------------------- ---- --- -----
    # Part Number          0x22   6 0J09D3    
    part_num = ''
    for l in b_info.split('\n'):
       if 'Part Number' in l:
           part_num = l.split()[4]
           break
    if part_num in b2f_part_nums:
       return 'B2F'        
    if part_num in f2b_part_nums:
       return 'F2B'        
    return None

class DellS6010PWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (DellS6010Fan1(),
            DellS6010Fan2(),
            DellS6010Fan3(),
            DellS6010Fan4(),
            DellS6010Fan5(),
            DellS6010Fan6(),
            DellS6010Fan7(),            
    )

    temps = (DellS6010Temp7(),
             DellS6010Temp8(),
             DellS6010Temp9(),
             DellS6010Temp10(),
             DellS6010T2PlusTemp1(),
             DellS6010T2PlusTemp2(),
             DellS6010T2PlusTemp3(),
             DellS6010T2PlusTemp4(),
             DellS6010T2PlusTemp5(),
             DellS6010T2PlusTemp6(),
             DellS6010T2PlusTemp7(),
             DellS6010T2PlusTemp8(),
             DellS6010T2PlusTemp9(),
             )
    f2b_temps = (DellS6010Temp1F2B(),
                 DellS6010Temp2F2B(),
                 DellS6010Temp3F2B(),
                 DellS6010Temp4F2B(),
                 DellS6010Temp5F2B(),
                 DellS6010Temp6F2B(),
                )
    b2f_temps = (DellS6010Temp1B2F(),
                 DellS6010Temp2B2F(),
                 DellS6010Temp3B2F(),
                 DellS6010Temp4B2F(),
                 DellS6010Temp5B2F(),
                 DellS6010Temp6B2F(),
                )
    # Default to B2F if airflow cant be figured out.
    if dell_s6010_airflow() == 'F2B':
        temps = temps + f2b_temps
    else:
        temps = temps + b2f_temps
        
class DellS6010Sensors():
    units = (DellS6010PSU1(),
             DellS6010PSU2(),
             DellS6010Temp7(),			  
             DellS6010Temp8(),
             DellS6010Temp9(),
             DellS6010Temp10(),
             DellS6010Fan1(),
             DellS6010Fan2(),
             DellS6010Fan3(),
             DellS6010Fan4(),
             DellS6010Fan5(),
             DellS6010Fan6(),
             DellS6010Fan7(),
             DellS6010T2PlusTemp1(),
             DellS6010T2PlusTemp2(),
             DellS6010T2PlusTemp3(),
             DellS6010T2PlusTemp4(),
             DellS6010T2PlusTemp5(),
             DellS6010T2PlusTemp6(),
             DellS6010T2PlusTemp7(),
             DellS6010T2PlusTemp8(),
             DellS6010T2PlusTemp9(),
    )

    f2b_temps = (DellS6010Temp1F2B(),
                 DellS6010Temp2F2B(),
                 DellS6010Temp3F2B(),
                 DellS6010Temp4F2B(),
                 DellS6010Temp5F2B(),
                 DellS6010Temp6F2B(),
                )
    b2f_temps = (DellS6010Temp1B2F(),
                 DellS6010Temp2B2F(),
                 DellS6010Temp3B2F(),
                 DellS6010Temp4B2F(),
                 DellS6010Temp5B2F(),
                 DellS6010Temp6B2F(),
                )
    # Default to B2F if airflow cant be figured out.
    if dell_s6010_airflow() == 'F2B':
        units = units + f2b_temps
    else:
        units = units + b2f_temps
        
class DellS6010PWMs():
    units = (DellS6010PWM1(),
             )

class DellS6010LEDs():
    units = (
             DellS6010SystemLED(),
             DellS6010PowerLED(),
             DellS6010FanLED(),
             DellS6010Fan1LED(),
             DellS6010Fan2LED(),
             DellS6010Fan3LED(),
             DellS6010Fan4LED(),
             DellS6010Fan5LED(),
        )

class DellS6010QSFPp(cumulus.platform.TridentTwoPlus40GPort):
    _set_led = '''
;;; DellS6010QSFPp
;;;
;;; 40G ports have 4 green/yellow LEDs, two bits per LED.
;;; For 40G mode, we use only the first LED to indicate link/activity.
;;;    0: link_green   (active high)
;;;    1-7: off
;;;  We use the link_green led for link and activity
;;; The other 3 LEDs will be off.
;;;
set_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor                     ; on with link
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ld      a, 4
qsfp_pack_loop:
        port 1
        pushst  PORTSTATUS_ZERO
        pack
        sub     a, 1
        jnz     qsfp_pack_loop

        ret
'''
    _set_led_4x10 = '''
;;; DellS6010QSFPp_4x10
;;;
;;;  For 4x10G mode, we use an LED for each port. Each port has
;;;  a single green/yellow LED:
;;;    0: link_green   (active high)
;;;    1: link_yellow  (active high)
;;;  We use the link_green led for link and activity
;;;  link_yellow is always off
;;
set_led:
        pushst  PORTSTATUS_ZERO
        pack
        call    set_qsfp_4x10_led
        inc     a
        call    set_qsfp_4x10_led
        inc     a
        call    set_qsfp_4x10_led
        inc     a
        call    set_qsfp_4x10_led

        ret

set_qsfp_4x10_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor                     ; link_green on with link
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ret
'''
    def __init__(self, label, serdes, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, leds):
        cumulus.platform.TridentTwoPlus40GPort.__init__(self,
                                                    label=label,
                                                    serdes=serdes,
                                                    phy_addrs = None,
                                                    phy_types = None,
                                                    tx_lane_map=tx_lane_map,
                                                    rx_lane_map=rx_lane_map,
                                                    tx_polarity_flip=tx_polarity_flip,
                                                    rx_polarity_flip=rx_polarity_flip,
                                                    leds=leds,
                                                    oversub=True,
                                                    unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode in ('4x1G', '4x10G')) and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        if mode in ('4x1G', '4x10G'):
            call = (fn_name + "_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None
        return set_calls

        
class DellS6010LED(cumulus.ledup.TridentTwoPlusLEDController):
    num_leds = 80

class DellS6010Switch(cumulus.platform.BCMSwitch):
    has_phy_ucode_spi_rom = False
    has_phy_ucode_mdio = False
    _QSFP = DellS6010QSFPp
    ledup = DellS6010LED

    ports = (    
    _QSFP(label="1",   serdes=(0, 1, 2, 3),     
		  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0xf,
		  rx_polarity_flip=0xf, leds=(0, 0),),
    _QSFP(label="2",   serdes=(4, 5, 6, 7),     
		  tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=0x0, 
		  rx_polarity_flip=0xf, leds=(0, 5),),
    _QSFP(label="3",   serdes=(8, 9, 10, 11),   
		  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0xf,
		  rx_polarity_flip=0xf, leds=(0, 10),),
    _QSFP(label="4",   serdes=(12, 13, 14, 15), 
		  tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=0x0,
		  rx_polarity_flip=0xf, leds=(0, 15),),
    _QSFP(label="5",   serdes=(16, 17, 18, 19), 
		  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0xf,
		  rx_polarity_flip=0x0, leds=(0, 20),),
    _QSFP(label="6",   serdes=(20, 21, 22, 23), 
		  tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=0x0,
		  rx_polarity_flip=0xf, leds=(0, 25),),
    _QSFP(label="7",   serdes=(24, 25, 26, 27), 
		  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0xf,
		  rx_polarity_flip=0x0, leds=(0, 30),),
    _QSFP(label="8",   serdes=(28, 29, 30, 31), 
		  tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=0x0,
		  rx_polarity_flip=0x0, leds=(0, 35),),
    _QSFP(label="9",   serdes=(32, 33, 34, 35), 
		  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0,
		  rx_polarity_flip=0x0, leds=(0, 40),),
    _QSFP(label="10",  serdes=(36, 37, 38, 39), 
		  tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=0xf,
		  rx_polarity_flip=0xf, leds=(0, 45),),
    _QSFP(label="11",  serdes=(40, 41, 42, 43), 
		  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0, 
		  rx_polarity_flip=0x0, leds=(0, 50),),
    _QSFP(label="12",  serdes=(44, 45, 46, 47), 
		  tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=0xf,
		  rx_polarity_flip=0xf, leds=(0, 55),),
    _QSFP(label="13",  serdes=(48, 49, 50, 51), 
		  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0,
		  rx_polarity_flip=0x0, leds=(0, 60),),
    _QSFP(label="14",  serdes=(52, 53, 54, 55), 
		  tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=0xf,
		  rx_polarity_flip=0xf, leds=(0, 65),),
    _QSFP(label="15",  serdes=(56, 57, 58, 59), 
		  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0,
		  rx_polarity_flip=0x0, leds=(0, 70),),
    _QSFP(label="16",  serdes=(60, 61, 62, 63), 
		  tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=0xf,
		  rx_polarity_flip=0xf, leds=(0, 75),),
    _QSFP(label="17",  serdes=(64, 65, 66, 67), 
		  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0, 
		  rx_polarity_flip=0x0, leds=(1, 0),),
    _QSFP(label="18",  serdes=(68, 69, 70, 71), 
		  tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=0xf,
		  rx_polarity_flip=0xf, leds=(1, 5),),
    _QSFP(label="19",  serdes=(72, 73, 74, 75), 
		  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0x0, 
		  rx_polarity_flip=0x0, leds=(1, 10),),
    _QSFP(label="20",  serdes=(76, 77, 78, 79), 
		  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0xa,
		  rx_polarity_flip=0x3, leds=(1, 15),),
    _QSFP(label="21",  serdes=(80, 81, 82, 83), 
		  tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=0x5,
		  rx_polarity_flip=0x0, leds=(1, 20),),
    _QSFP(label="22",  serdes=(84, 85, 86, 87), 
		  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0xa, 
		  rx_polarity_flip=0x0, leds=(1, 25),),
    _QSFP(label="23",  serdes=(88, 89, 90, 91), 
		  tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=0x5, 
		  rx_polarity_flip=0x0, leds=(1, 30),),
    _QSFP(label="24",  serdes=(92, 93, 94, 95), 
		  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0xa,
		  rx_polarity_flip=0x0, leds=(1, 35),),
    _QSFP(label="25",  serdes=(96, 97, 98, 99), 
		  tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=0x5,
		  rx_polarity_flip=0x0, leds=(1, 40),),
    _QSFP(label="26",  serdes=(100, 101, 102, 103), 
		  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0,
		  rx_polarity_flip=0x0, leds=(1, 45),),
    _QSFP(label="27",  serdes=(104, 105, 106, 107), 
		  tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=0xf,
		  rx_polarity_flip=0x0, leds=(1, 50),),
    _QSFP(label="28",  serdes=(108, 109, 110, 111), 
		  tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=0x0,
		  rx_polarity_flip=0x0, leds=(1, 55),),
    _QSFP(label="29",  serdes=(112, 113, 114, 115), 
		  tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=0xf,
		  rx_polarity_flip=0x0, leds=(1, 60),),
    _QSFP(label="30",  serdes=(116, 117, 118, 119), 
		  tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=0x0,
		  rx_polarity_flip=0xf, leds=(1, 65),),
    _QSFP(label="31",  serdes=(120, 121, 122, 123), 
		  tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=0xf,
		  rx_polarity_flip=0x0, leds=(1, 70),),
    _QSFP(label="32",  serdes=(124, 125, 126, 127), 
		  tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=0x0, 
		  rx_polarity_flip=0x0, leds=(1, 75),),    
    )
        
class DellS6010(cumulus.platform.Platform):
    name = 'dell,s6010_c2538'
    chip = cumulus.platform.TridentTwoPlus_56860_Chip()
    switch = DellS6010Switch(chip)
    cpld = DellS6010CPLD()
    pwms = DellS6010PWMs()
    sensors = DellS6010Sensors()
    statusleds = DellS6010LEDs()

Platform = DellS6010
