#
# Copyright (c) 2019-2020 Cumulus Networks,  Inc.  All rights reserved.
#

import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
#from cumulus.platforms.dellemc_s52xx_sensors import *

class DellS5212FSFP28(cumulus.platform.TridentThree25GPort):

    def __init__(self, label, serdes, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp28')
        cumulus.platform.TridentThree25GPort.__init__(self,
                                                  label=label,
                                                  serdes=serdes,
                                                  leds=None,
                                                  phy_types=None,
                                                  phy_addrs=None,
                                                  phy_layer=sfp.type.upper(),
                                                  tx_lane_map=tx_lane_map,
                                                  rx_lane_map=rx_lane_map,
                                                  tx_polarity_flip=tx_polarity_flip,
                                                  rx_polarity_flip=rx_polarity_flip,
                                                  pre_emphasis=pre_emphasis,
                                                  unit=0,
                                                  oversub=True)
        self.sfp = sfp

    def logical_port_num(self, mode, sub, port_base):
        lport = self.hw_intf_num(mode, sub) + 1
        reMap = { 29: 1,
                  30: 2,
                  31: 3,
                  32: 4,
                  33: 5,
                  34: 6,
                  35: 7,
                  36: 8,
                  37: 9,
                  38: 10,
                  39: 11,
                  40: 12,
                  }
        lport = reMap.get(lport, lport)
        return lport

class DellS5212FQSFP28(cumulus.platform.TridentThree100GPort):

    def __init__(self, label, serdes, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.TridentThree100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   leds=None,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   tx_lane_map=tx_lane_map,
                                                   rx_lane_map=rx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   unit=0,
                                                   oversub=True)
        self.sfp = sfp

    def logical_port_num(self, mode, sub, port_base):
        lport = self.hw_intf_num(mode, sub) + 1
        reMap = { 41: 33,
                  42: 34,
                  43: 35,
                  44: 36,
                  45: 37,
                  46: 38,
                  47: 39,
                  48: 40,
                  49: 41,
                  50: 42,
                  51: 43,
                  52: 44,
                  }
        lport = reMap.get(lport, lport)
        return lport

# Dell S5212F switch ports

class DellS5212FSwitch(cumulus.platform.BCMSwitch):
    _sfp28 = DellS5212FSFP28
    _qsfp28 = DellS5212FQSFP28
    has_phy_ucode_spi_rom = False
    has_phy_ucode_mdio = False

    ports = (
        # FC7
        _sfp28(label="1",  serdes=( 28,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="2",  serdes=( 29,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="3",  serdes=( 30,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="4",  serdes=( 31,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC8
        _sfp28(label="5",  serdes=( 32,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="6",  serdes=( 33,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="7",  serdes=( 34,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="8",  serdes=( 35,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC9
        _sfp28(label="9",  serdes=( 36,), tx_lane_map=0x2013, rx_lane_map=0x0231, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="10", serdes=( 37,), tx_lane_map=0x2013, rx_lane_map=0x0231, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="11", serdes=( 38,), tx_lane_map=0x2013, rx_lane_map=0x0231, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="12", serdes=( 39,), tx_lane_map=0x2013, rx_lane_map=0x0231, tx_polarity_flip=False, rx_polarity_flip=False),

        # FC10
        _qsfp28(label="13", serdes=(40, 41, 42, 43), tx_lane_map=0x2301, rx_lane_map=0x1230, tx_polarity_flip=0x6, rx_polarity_flip=0xb),

        # FC11
        _qsfp28(label="14", serdes=(44, 45, 46, 47), tx_lane_map=0x3120, rx_lane_map=0x0213, tx_polarity_flip=0x3, rx_polarity_flip=0x3),

        # FC12
        _qsfp28(label="15", serdes=(48, 49, 50, 51), tx_lane_map=0x2301, rx_lane_map=0x1032, tx_polarity_flip=0xe, rx_polarity_flip=0x8),
)

vhwmon_path = '/sys/class/misc/vhwmon'

# PSUs
class DellS5212PSU(cumulus.sensors.PSU_Unit):
    driver_path = '/sys/devices/pci0000:00/0000:00:0c.0/0000:03:00.0'
    psu_ok = 0

class DellS5212PSU1(DellS5212PSU):
    name = 'PSU1'
    description = 'PSU1'
    driver_hwmon = ['psu_pwr1']

    def get_status(self):
        val = self.attrs.get('psu_pwr1_all_ok', 0)
        self.attrs['psu_pwr1_all_ok'] = val
        DellS5212PSU1.psu_ok = int(val)
        return cumulus.sensors.PSU_Unit.get_status(self)

class DellS5212PSU2(DellS5212PSU):
    name = 'PSU2'
    description = 'PSU2'
    driver_hwmon = ['psu_pwr2']

    def get_status(self):
        val = self.attrs.get('psu_pwr2_all_ok', 0)
        self.attrs['psu_pwr2_all_ok'] = val
        DellS5212PSU2.psu_ok = int(val)
        return cumulus.sensors.PSU_Unit.get_status(self)

# Intel Denverton CPU
class DellS5212CoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    no_shutdown = True

class DellS5212Temp1(DellS5212CoreTemp):
    name = 'Temp1'
    description = 'Physical id 0'
    driver_hwmon = ['temp1']

class DellS5212Temp2(DellS5212CoreTemp):
    name = 'Temp2'
    description = 'Core 6'
    driver_hwmon = ['temp8']

class DellS5212Temp3(DellS5212CoreTemp):
    name = 'Temp3'
    description = 'Core 12'
    driver_hwmon = ['temp14']

# Trident3 Sensors
class DellS5212ASICTemp(cumulus.sensors.Temp_Unit):
    driver_path = '/cumulus/switchd/run'
    description = cumulus.sensors.Temp_Unit.mac_die_sensor_name

class DellS5212Temp4(DellS5212ASICTemp):
    name = 'Temp4'
    driver_hwmon = ['temp1']

class DellS5212Temp5(DellS5212ASICTemp):
    name = 'Temp5'
    driver_hwmon = ['temp2']

class DellS5212Temp6(DellS5212ASICTemp):
    name = 'Temp6'
    driver_hwmon = ['temp3']

class DellS5212Temp7(DellS5212ASICTemp):
    name = 'Temp7'
    driver_hwmon = ['temp4']

class DellS5212Temp8(DellS5212ASICTemp):
    name = 'Temp8'
    driver_hwmon = ['temp5']

class DellS5212Temp9(DellS5212ASICTemp):
    name = 'Temp9'
    driver_hwmon = ['temp6']

class DellS5212Temp10(DellS5212ASICTemp):
    name = 'Temp10'
    driver_hwmon = ['temp7']

class DellS5212Temp11(DellS5212ASICTemp):
    name = 'Temp11'
    driver_hwmon = ['temp8']

class DellS5212Temp12(DellS5212ASICTemp):
    name = 'Temp12'
    driver_hwmon = ['temp9']

class DellS5212Temp13(DellS5212ASICTemp):
    name = 'Temp13'
    driver_hwmon = ['temp10']

# BMC Sensors
class DellS5212BMCTemp(cumulus.sensors.Temp_Unit):
    driver_path = vhwmon_path + '/bmc:temp'

class DellS5212Temp14(DellS5212BMCTemp):
    name = 'Temp14'
    description = 'PT_Mid_temp'
    driver_hwmon = ['temp1']
    
class DellS5212Temp15(DellS5212BMCTemp):
    name = 'Temp15'
    description = 'NPU_Near_temp'
    driver_hwmon = ['temp2']
    
class DellS5212Temp16(DellS5212BMCTemp):
    name = 'Temp16'
    description = 'PT_Left_temp'
    driver_hwmon = ['temp3']

class DellS5212Temp17(DellS5212BMCTemp):
    name = 'Temp17'
    description = 'PT_Right_temp'
    driver_hwmon = ['temp4']
    
class DellS5212Temp18(DellS5212BMCTemp):
    name = 'Temp18'
    description = 'ILET_AF_temp'
    driver_hwmon = ['temp5']
    
class DellS5212Temp19(DellS5212BMCTemp):
    name = 'Temp19'
    description = 'CPU_temp'
    driver_hwmon = ['temp6']
    
# Fans
class DellS5212Fan(cumulus.sensors.Fan_Unit):
    driver_path = vhwmon_path + '/bmc:fan'
    no_pwm = True

class DellS5212Fan1(DellS5212Fan):
    name = 'Fan1'
    description = 'FAN1_Rear_rpm'
    driver_hwmon = ['fan1']

class DellS5212Fan2(DellS5212Fan):
    name = 'Fan2'
    description = 'FAN2_Rear_rpm'
    driver_hwmon = ['fan2']

class DellS5212Fan3(DellS5212Fan):
    name = 'Fan3'
    description = 'FAN3_Rear_rpm'
    driver_hwmon = ['fan3']

class DellS5212Fan4(DellS5212Fan):
    name = 'Fan4'
    description = 'FAN4_Rear_rpm'
    driver_hwmon = ['fan4']

class DellS5212Fan5(DellS5212Fan):
    name = 'Fan5'
    description = 'FAN1_Front_rpm'
    driver_hwmon = ['fan5']

class DellS5212Fan6(DellS5212Fan):
    name = 'Fan6'
    description = 'FAN2_Front_rpm'
    driver_hwmon = ['fan6']

class DellS5212Fan7(DellS5212Fan):
    name = 'Fan7'
    description = 'FAN3_Front_rpm'
    driver_hwmon = ['fan7']

class DellS5212Fan8(DellS5212Fan):
    name = 'Fan8'
    description = 'FAN4_Front_rpm'
    driver_hwmon = ['fan8']

# LEDs

class DellS5212SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_path = '/sys/devices/pci0000:00/0000:00:0c.0/0000:03:00.0'
    led_location = 'led_system'
    good_led_color = 'green'
    bad_led_color = 'amber'

class DellS5212PowerLED(cumulus.sensors.StatusLED_Unit):
    name = 'Power'
    sources = ['PSU1', 'PSU2']
    led_path = '/sys/devices/pci0000:00/0000:00:0c.0/0000:03:00.0'
    led_location = 'led_power'
    good_led_color = 'green'
    bad_led_color = 'amber_blinking'

class DellS5212LEDs():
    units = ( DellS5212SystemLED(),
              DellS5212PowerLED(),
              )

class DellS5212BMC(cumulus.platform.BMC):
    name = 'bmc'
    type = 'ipmibmc'

class DellS5212FSensors():
    units = (DellS5212PSU1(),
             DellS5212PSU2(),
             DellS5212Temp1(),
             DellS5212Temp2(),
             DellS5212Temp3(),
             DellS5212Temp4(),
             DellS5212Temp5(),
             DellS5212Temp6(),
             DellS5212Temp7(),
             DellS5212Temp8(),
             DellS5212Temp9(),
             DellS5212Temp10(),
             DellS5212Temp11(),
             DellS5212Temp12(),
             DellS5212Temp13(),
             DellS5212Temp14(),
             DellS5212Temp15(),
             DellS5212Temp16(),
             DellS5212Temp17(),
             DellS5212Temp18(),
             DellS5212Temp19(),
             DellS5212Fan1(),
             DellS5212Fan2(),
             DellS5212Fan3(),
             DellS5212Fan4(),
             DellS5212Fan5(),
             DellS5212Fan6(),
             DellS5212Fan7(),
             DellS5212Fan8(),
             )

class DellS5212F(cumulus.platform.Platform):
    name = 'dellemc,s5212f_c3538'
    chip = cumulus.platform.TridentThreeX5_56771_Chip()
    switch = DellS5212FSwitch(chip)
    cpld = None
    bmc = DellS5212BMC()
    sensors = DellS5212FSensors()
    statusleds = DellS5212LEDs()
    pwms = None

Platform = DellS5212F
