#
# Copyright (c) 2019 Cumulus Networks, Inc.  All rights reserved.
#

import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
from cumulus.platforms.dellemc_s52xx_sensors import *

class DellS5224FSFP28(cumulus.platform.TridentThree25GPort):

    def __init__(self, label, serdes, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp28')
        cumulus.platform.TridentThree25GPort.__init__(self,
                                                  label=label,
                                                  serdes=serdes,
                                                  leds=None,
                                                  phy_types=None,
                                                  phy_addrs=None,
                                                  phy_layer=sfp.type.upper(),
                                                  tx_lane_map=tx_lane_map,
                                                  rx_lane_map=rx_lane_map,
                                                  tx_polarity_flip=tx_polarity_flip,
                                                  rx_polarity_flip=rx_polarity_flip,
                                                  pre_emphasis=pre_emphasis,
                                                  unit=0,
                                                  oversub=True)
        self.sfp = sfp

class DellS5224FQSFP28(cumulus.platform.TridentThree100GPort):

    def __init__(self, label, serdes, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.TridentThree100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   leds=None,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   tx_lane_map=tx_lane_map,
                                                   rx_lane_map=rx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   unit=0,
                                                   oversub=True)
        self.sfp = sfp

# Dell S5224F switch ports

class DellS5224FSwitch(cumulus.platform.BCMSwitch):
    _sfp28 = DellS5224FSFP28
    _qsfp28 = DellS5224FQSFP28
    has_phy_ucode_spi_rom = False
    has_phy_ucode_mdio = False

    ports = (
        # FC1
        _sfp28(label="1",  serdes=(  4,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="2",  serdes=(  5,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="3",  serdes=(  6,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="4",  serdes=(  7,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC3
        _sfp28(label="5",  serdes=( 12,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="6",  serdes=( 13,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="7",  serdes=( 14,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="8",  serdes=( 15,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC4
        _sfp28(label="9",  serdes=( 16,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="10", serdes=( 17,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="11", serdes=( 18,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="12", serdes=( 19,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True ),

        # FC9
        _sfp28(label="13", serdes=( 36,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="14", serdes=( 37,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="15", serdes=( 38,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="16", serdes=( 39,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC11
        _sfp28(label="17", serdes=( 44,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="18", serdes=( 45,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="19", serdes=( 46,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="20", serdes=( 47,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC13
        _sfp28(label="21", serdes=( 52,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="22", serdes=( 53,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="23", serdes=( 54,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="24", serdes=( 55,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC7
        _qsfp28(label="25",  serdes=( 28,  29,  30,  31), tx_lane_map=0x0123, rx_lane_map=0x3201, tx_polarity_flip=0x3, rx_polarity_flip=0x2),

        # FC5
        _qsfp28(label="26",  serdes=( 20,  21,  22,  23), tx_lane_map=0x3021, rx_lane_map=0x0312, tx_polarity_flip=0x4, rx_polarity_flip=0xf),

        # FC31
        _qsfp28(label="27",  serdes=(124, 125, 126, 127), tx_lane_map=0x3210, rx_lane_map=0x0132, tx_polarity_flip=0x1, rx_polarity_flip=0x8),

        # FC21
        _qsfp28(label="28", serdes=( 84,  85,  86,  87), tx_lane_map=0x0312, rx_lane_map=0x3021, tx_polarity_flip=0x4, rx_polarity_flip=0x4),
)

class DellS5224FSensors(DellS52xxSensors):
    units = (DellS52xxPSU1(),
             DellS52xxPSU2(),
             DellS52xxTemp1(),
             DellS52xxTemp2(),
             DellS52xxTemp3(),
             DellS52xxTemp4(),
             DellS52xxTemp5(),
             DellS52xxTemp6(),
             DellS52xxTemp7(),
             DellS52xxTemp8(),
             DellS52xxTemp9(),
             DellS52xxTemp10(),
             DellS52xxTemp11(),
             DellS52xxTemp12(),
             DellS52xxTemp13(),
             DellS52xxTemp14(),
             DellS52xxTemp15(),
             DellS52xxTemp16(),
             DellS52xxTemp17(),
             DellS52xxTemp18(),
             DellS52xxTemp19(),
             DellS52xxTemp20(),
             DellS52xxTemp21(),
             DellS52xxTemp22(),
             DellS52xxTemp23(),
             DellS52xxTemp24(),
             DellS52xxTemp25(),
             DellS52xxTemp26(),
             DellS52xxTemp27(),
             DellS52xxFan1(),
             DellS52xxFan2(),
             DellS52xxFan3(),
             DellS52xxFan4(),
             DellS52xxFan5(),
             DellS52xxFan6(),
             DellS52xxFan7(),
             DellS52xxFan8(),
             DellS52xxPSU1Fan1(),
             DellS52xxPSU2Fan1(),
             )

class DellS5224F(cumulus.platform.Platform):
    name = 'dellemc,s5224f_c3538'
    chip = cumulus.platform.TridentThreeX5_56771_Chip()
    switch = DellS5224FSwitch(chip)
    cpld = None
    bmc = DellS52xxBMC()
    sensors = DellS5224FSensors()
    statusleds = DellS52xxLEDs()
    pwms = None

Platform = DellS5224F
