#
# Copyright (c) 2018 Cumulus Networks, Inc.  All rights reserved.
#

import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
from cumulus.platforms.dellemc_s52xx_sensors import *

class DellS5232FQSFP28(cumulus.platform.TridentThree100GPort):

    def __init__(self, label, serdes, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.TridentThree100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   leds=None,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   tx_lane_map=tx_lane_map,
                                                   rx_lane_map=rx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   unit=0,
                                                   oversub=True)
        self.sfp = sfp

# Dell S5232F switch ports

class DellS5232FSwitch(cumulus.platform.BCMSwitch):
    _qsfp28 = DellS5232FQSFP28
    has_phy_ucode_spi_rom = False
    has_phy_ucode_mdio = False

    ports = (

        _qsfp28(label="1",  serdes=(  0,   1,   2,   3), tx_lane_map=0x0132, rx_lane_map=0x3210, tx_polarity_flip=0x8, rx_polarity_flip=0xa),
        _qsfp28(label="2",  serdes=(  4,   5,   6,   7), tx_lane_map=0x2301, rx_lane_map=0x2031, tx_polarity_flip=0xa, rx_polarity_flip=0xe),
        _qsfp28(label="3",  serdes=(  8,   9,  10,  11), tx_lane_map=0x0132, rx_lane_map=0x3210, tx_polarity_flip=0x8, rx_polarity_flip=0xa),
        _qsfp28(label="4",  serdes=( 12,  13,  14,  15), tx_lane_map=0x3201, rx_lane_map=0x2031, tx_polarity_flip=0x2, rx_polarity_flip=0xe),
        _qsfp28(label="5",  serdes=( 16,  17,  18,  19), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0xa, rx_polarity_flip=0xa),
        _qsfp28(label="6",  serdes=( 20,  21,  22,  23), tx_lane_map=0x2301, rx_lane_map=0x2031, tx_polarity_flip=0xa, rx_polarity_flip=0xe),
        _qsfp28(label="7",  serdes=( 24,  25,  26,  27), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0xa, rx_polarity_flip=0x5),
        _qsfp28(label="8",  serdes=( 28,  29,  30,  31), tx_lane_map=0x3201, rx_lane_map=0x2031, tx_polarity_flip=0x3, rx_polarity_flip=0x1),

        _qsfp28(label="9",  serdes=( 32,  33,  34,  35), tx_lane_map=0x0213, rx_lane_map=0x1302, tx_polarity_flip=0x9, rx_polarity_flip=0x1),
        _qsfp28(label="10", serdes=( 36,  37,  38,  39), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x7, rx_polarity_flip=0x9),
        _qsfp28(label="11", serdes=( 40,  41,  42,  43), tx_lane_map=0x0231, rx_lane_map=0x3120, tx_polarity_flip=0xd, rx_polarity_flip=0xb),
        _qsfp28(label="12", serdes=( 44,  45,  46,  47), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x7, rx_polarity_flip=0xc),
        _qsfp28(label="13", serdes=( 48,  49,  50,  51), tx_lane_map=0x2103, rx_lane_map=0x3120, tx_polarity_flip=0xb, rx_polarity_flip=0xc),
        _qsfp28(label="14", serdes=( 52,  53,  54,  55), tx_lane_map=0x2301, rx_lane_map=0x2031, tx_polarity_flip=0xa, rx_polarity_flip=0xe),
        _qsfp28(label="15", serdes=( 56,  57,  58,  59), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=0xb, rx_polarity_flip=0xa),
        _qsfp28(label="16", serdes=( 60,  61,  62,  63), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=0x2, rx_polarity_flip=0x5),

        _qsfp28(label="17", serdes=( 64,  65,  66,  67), tx_lane_map=0x3210, rx_lane_map=0x1023, tx_polarity_flip=0x5, rx_polarity_flip=0x4),
        _qsfp28(label="18", serdes=( 68,  69,  70,  71), tx_lane_map=0x0123, rx_lane_map=0x1302, tx_polarity_flip=0x5, rx_polarity_flip=0x1),
        _qsfp28(label="19", serdes=( 72,  73,  74,  75), tx_lane_map=0x2301, rx_lane_map=0x1032, tx_polarity_flip=0xa, rx_polarity_flip=0x5),
        _qsfp28(label="20", serdes=( 76,  77,  78,  79), tx_lane_map=0x2013, rx_lane_map=0x3120, tx_polarity_flip=0xc, rx_polarity_flip=0xc),
        _qsfp28(label="21", serdes=( 80,  81,  82,  83), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0xc, rx_polarity_flip=0x4),
        _qsfp28(label="22", serdes=( 84,  85,  86,  87), tx_lane_map=0x0123, rx_lane_map=0x2130, tx_polarity_flip=0x5, rx_polarity_flip=0x3),
        _qsfp28(label="23", serdes=( 88,  89,  90,  91), tx_lane_map=0x2301, rx_lane_map=0x2031, tx_polarity_flip=0x5, rx_polarity_flip=0xc),
        _qsfp28(label="24", serdes=( 92,  93,  94,  95), tx_lane_map=0x0312, rx_lane_map=0x2310, tx_polarity_flip=0x3, rx_polarity_flip=0xb),

        _qsfp28(label="25", serdes=( 96,  97,  98,  99), tx_lane_map=0x2301, rx_lane_map=0x1032, tx_polarity_flip=0x5, rx_polarity_flip=0x5),
        _qsfp28(label="26", serdes=(100, 101, 102, 103), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0x5, rx_polarity_flip=0x2),
        _qsfp28(label="27", serdes=(104, 105, 106, 107), tx_lane_map=0x2301, rx_lane_map=0x1032, tx_polarity_flip=0x5, rx_polarity_flip=0xa),
        _qsfp28(label="28", serdes=(108, 109, 110, 111), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0x5, rx_polarity_flip=0x5),
        _qsfp28(label="29", serdes=(112, 113, 114, 115), tx_lane_map=0x2301, rx_lane_map=0x2031, tx_polarity_flip=0x5, rx_polarity_flip=0x1),
        _qsfp28(label="30", serdes=(116, 117, 118, 119), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0x5, rx_polarity_flip=0x5),
        _qsfp28(label="31", serdes=(120, 121, 122, 123), tx_lane_map=0x2301, rx_lane_map=0x1032, tx_polarity_flip=0x5, rx_polarity_flip=0xa),
        _qsfp28(label="32", serdes=(124, 125, 126, 127), tx_lane_map=0x0123, rx_lane_map=0x3210, tx_polarity_flip=0x5, rx_polarity_flip=0x5),
)

class DellS5232F(cumulus.platform.Platform):
    name = 'dellemc,s5232f_c2538'
    chip = cumulus.platform.TridentThreeX7_56870_Chip()
    switch = DellS5232FSwitch(chip)
    cpld = None
    bmc = DellS52xxBMC()
    sensors = DellS52xxSensors()
    statusleds = DellS52xxLEDs()
    pwms = None

Platform = DellS5232F
