#
# Copyright (c) 2019 Cumulus Networks, Inc.  All rights reserved.
#

import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
from cumulus.platforms.dellemc_s52xx_sensors import *

class DellS5296FSFP28(cumulus.platform.TridentThree25GPort):

    def __init__(self, label, serdes, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp28')
        cumulus.platform.TridentThree25GPort.__init__(self,
                                                  label=label,
                                                  serdes=serdes,
                                                  leds=None,
                                                  phy_types=None,
                                                  phy_addrs=None,
                                                  phy_layer=sfp.type.upper(),
                                                  tx_lane_map=tx_lane_map,
                                                  rx_lane_map=rx_lane_map,
                                                  tx_polarity_flip=tx_polarity_flip,
                                                  rx_polarity_flip=rx_polarity_flip,
                                                  pre_emphasis=pre_emphasis,
                                                  unit=0,
                                                  oversub=True)
        self.sfp = sfp

class DellS5296FQSFP28(cumulus.platform.TridentThree100GPort):

    def __init__(self, label, serdes, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.TridentThree100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   leds=None,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   tx_lane_map=tx_lane_map,
                                                   rx_lane_map=rx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   unit=0,
                                                   oversub=True)
        self.sfp = sfp

# Dell S5296F switch ports

class DellS5296FSwitch(cumulus.platform.BCMSwitch):
    _sfp28 = DellS5296FSFP28
    _qsfp28 = DellS5296FQSFP28
    has_phy_ucode_spi_rom = False
    has_phy_ucode_mdio = False

    ports = (
        # FC1
        _sfp28(label="1",  serdes=(  4,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="2",  serdes=(  5,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="3",  serdes=(  6,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="4",  serdes=(  7,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC3
        _sfp28(label="5",  serdes=( 12,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="6",  serdes=( 13,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="7",  serdes=( 14,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="8",  serdes=( 15,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC4
        _sfp28(label="9",  serdes=( 16,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="10", serdes=( 17,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="11", serdes=( 18,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="12", serdes=( 19,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True ),

        # FC9
        _sfp28(label="13", serdes=( 36,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="14", serdes=( 37,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="15", serdes=( 38,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="16", serdes=( 39,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC11
        _sfp28(label="17", serdes=( 44,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="18", serdes=( 45,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="19", serdes=( 46,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="20", serdes=( 47,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC13
        _sfp28(label="21", serdes=( 52,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="22", serdes=( 53,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="23", serdes=( 54,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="24", serdes=( 55,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC16
        _sfp28(label="25", serdes=( 64,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="26", serdes=( 65,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="27", serdes=( 66,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="28", serdes=( 67,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC18
        _sfp28(label="29", serdes=( 72,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="30", serdes=( 73,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="31", serdes=( 74,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="32", serdes=( 75,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC22
        _sfp28(label="33", serdes=( 88,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="34", serdes=( 89,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="35", serdes=( 90,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="36", serdes=( 91,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True ),

        # FC25
        _sfp28(label="37", serdes=(100,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="38", serdes=(101,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="39", serdes=(102,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="40", serdes=(103,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False),

        # FC27
        _sfp28(label="41", serdes=(108,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="42", serdes=(109,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="43", serdes=(110,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="44", serdes=(111,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC29
        _sfp28(label="45", serdes=(116,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="46", serdes=(117,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="47", serdes=(118,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="48", serdes=(119,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC0
        _sfp28(label="49", serdes=(  0,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="50", serdes=(  1,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="51", serdes=(  2,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="52", serdes=(  3,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC2
        _sfp28(label="53", serdes=(  8,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="54", serdes=(  9,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="55", serdes=( 10,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="56", serdes=( 11,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC6
        _sfp28(label="57", serdes=( 24,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="58", serdes=( 25,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="59", serdes=( 26,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="60", serdes=( 27,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),

        # FC8
        _sfp28(label="61", serdes=( 32,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="62", serdes=( 33,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="63", serdes=( 34,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="64", serdes=( 35,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),

        # FC10
        _sfp28(label="65", serdes=( 40,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="66", serdes=( 41,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="67", serdes=( 42,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="68", serdes=( 43,), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),

        # FC12
        _sfp28(label="69", serdes=( 48,), tx_lane_map=0x2031, rx_lane_map=0x2031, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="70", serdes=( 49,), tx_lane_map=0x2031, rx_lane_map=0x2031, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="71", serdes=( 50,), tx_lane_map=0x2031, rx_lane_map=0x2031, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="72", serdes=( 51,), tx_lane_map=0x2031, rx_lane_map=0x2031, tx_polarity_flip=False, rx_polarity_flip=False),

        # FC17
        _sfp28(label="73", serdes=( 68,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="74", serdes=( 69,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="75", serdes=( 70,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="76", serdes=( 71,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),

        # FC19
        _sfp28(label="77", serdes=( 76,), tx_lane_map=0x1302, rx_lane_map=0x1302, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="78", serdes=( 77,), tx_lane_map=0x1302, rx_lane_map=0x1302, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="79", serdes=( 78,), tx_lane_map=0x1302, rx_lane_map=0x1302, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="80", serdes=( 79,), tx_lane_map=0x1302, rx_lane_map=0x1302, tx_polarity_flip=False, rx_polarity_flip=False),

        # FC20
        _sfp28(label="81", serdes=( 80,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="82", serdes=( 81,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="83", serdes=( 82,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="84", serdes=( 83,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC24
        _sfp28(label="85", serdes=( 96,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="86", serdes=( 97,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="87", serdes=( 98,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="88", serdes=( 99,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),

        # FC26
        _sfp28(label="89", serdes=(104,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="90", serdes=(105,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="91", serdes=(106,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="92", serdes=(107,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),

        # FC28
        _sfp28(label="93", serdes=(112,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="94", serdes=(113,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="95", serdes=(114,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="96", serdes=(115,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),

        # FC7
        _qsfp28(label="97",  serdes=( 28,  29,  30,  31), tx_lane_map=0x0123, rx_lane_map=0x3201, tx_polarity_flip=0x3, rx_polarity_flip=0x2),

        # FC5
        _qsfp28(label="98",  serdes=( 20,  21,  22,  23), tx_lane_map=0x3021, rx_lane_map=0x0312, tx_polarity_flip=0x4, rx_polarity_flip=0xf),

        # FC31
        _qsfp28(label="99",  serdes=(124, 125, 126, 127), tx_lane_map=0x3210, rx_lane_map=0x0132, tx_polarity_flip=0x1, rx_polarity_flip=0x8),

        # FC21
        _qsfp28(label="100", serdes=( 84,  85,  86,  87), tx_lane_map=0x0312, rx_lane_map=0x3021, tx_polarity_flip=0x4, rx_polarity_flip=0x4),

        # FC14
        _qsfp28(label="101", serdes=( 56,  57,  58,  59), tx_lane_map=0x3021, rx_lane_map=0x0312, tx_polarity_flip=0x4, rx_polarity_flip=0x2),

        # FC15
        _qsfp28(label="102", serdes=( 60,  61,  62,  63), tx_lane_map=0x3210, rx_lane_map=0x1230, tx_polarity_flip=0xe, rx_polarity_flip=0xd),

        # FC30
        _qsfp28(label="103", serdes=(120, 121, 122, 123), tx_lane_map=0x0123, rx_lane_map=0x2310, tx_polarity_flip=0xf, rx_polarity_flip=0xe),

        # FC23
        _qsfp28(label="104", serdes=( 92,  93,  94,  95), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=0x4, rx_polarity_flip=0x9),
)

class DellS5296FSensors(DellS52xxSensors):
    units = (DellS52xxPSU1(),
             DellS52xxPSU2(),
             DellS52xxTemp1(),
             DellS52xxTemp2(),
             DellS52xxTemp3(),
             DellS52xxTemp4(),
             DellS52xxTemp5(),
             DellS52xxTemp6(),
             DellS52xxTemp7(),
             DellS52xxTemp8(),
             DellS52xxTemp9(),
             DellS52xxTemp10(),
             DellS52xxTemp11(),
             DellS52xxTemp12(),
             DellS52xxTemp13(),
             DellS52xxTemp14(),
             DellS52xxTemp15(),
             DellS52xxTemp16(),
             DellS52xxTemp17(),
             DellS52xxTemp18(),
             DellS52xxTemp19(),
             DellS52xxTemp20(),
             DellS52xxTemp21(),
             DellS52xxTemp22(),
             DellS52xxTemp23(),
             DellS52xxTemp24(),
             DellS52xxTemp25(),
             DellS52xxTemp26(),
             DellS52xxTemp27(),
             DellS52xxFan1(),
             DellS52xxFan2(),
             DellS52xxFan3(),
             DellS52xxFan4(),
             DellS52xxPSU1Fan1(),
             DellS52xxPSU2Fan1(),
             )

class DellS5296F(cumulus.platform.Platform):
    name = 'dellemc,s5296f_c3538'
    chip = cumulus.platform.TridentThreeX7_56870_Chip()
    switch = DellS5296FSwitch(chip)
    cpld = None
    bmc = DellS52xxBMC()
    sensors = DellS5296FSensors()
    statusleds = DellS52xxLEDs()
    pwms = None

Platform = DellS5296F
