#
# Copyright (c) 2018-2020 Cumulus Networks,  Inc.  All rights reserved.
#

import os
import sys
import string
import subprocess
import cumulus.platform
import cumulus.ledup
import cumulus.sunit
import cumulus.sensors

vhwmon_path = '/sys/class/misc/vhwmon'

# PSUs
class DellS52xxPSU(cumulus.sensors.PSU_Unit):
    driver_path = '/sys/devices/pci0000:00/0000:00:0c.0/0000:04:00.0'
    psu_ok = 0

class DellS52xxPSU1(DellS52xxPSU):
    name = 'PSU1'
    description = 'PSU1'
    driver_hwmon = ['psu_pwr1']

    def get_status(self):
        val = self.attrs.get('psu_pwr1_all_ok', 0)
        self.attrs['psu_pwr1_all_ok'] = val
        DellS52xxPSU1.psu_ok = int(val)
        return cumulus.sensors.PSU_Unit.get_status(self)

class DellS52xxPSU2(DellS52xxPSU):
    name = 'PSU2'
    description = 'PSU2'
    driver_hwmon = ['psu_pwr2']

    def get_status(self):
        val = self.attrs.get('psu_pwr2_all_ok', 0)
        self.attrs['psu_pwr2_all_ok'] = val
        DellS52xxPSU2.psu_ok = int(val)
        return cumulus.sensors.PSU_Unit.get_status(self)

# Intel Denverton CPU
class DellS52xxCoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    no_shutdown = True

    hwmon_path = driver_path + "/hwmon/"
    hwmon_dirs = os.listdir(hwmon_path)
    driver_hwmons = []
    descriptions = []

    for dir in hwmon_dirs:
        files = os.listdir(hwmon_path + dir)
        for file in files:
            if str.endswith(file,'_label'):
                idx = string.find(file,"_")
                tempx = file[:idx]
                driver_hwmons.append(tempx)

                file_path = hwmon_path + dir + '/' + file
                f = open(file_path,'r')
                description = f.read()
                f.close()
                description = string.rstrip(description,'\n')
                descriptions.append(description) 

class DellS52xxTemp1(DellS52xxCoreTemp):
    name = 'Temp1'
    driver_hwmon = []
    driver_hwmon.append(DellS52xxCoreTemp.driver_hwmons[0])
    description = DellS52xxCoreTemp.descriptions[0]

class DellS52xxTemp2(DellS52xxCoreTemp):
    name = 'Temp2'
    driver_hwmon = []
    driver_hwmon.append(DellS52xxCoreTemp.driver_hwmons[1])
    description = DellS52xxCoreTemp.descriptions[1]

class DellS52xxTemp3(DellS52xxCoreTemp):
    name = 'Temp3'
    driver_hwmon = []
    driver_hwmon.append(DellS52xxCoreTemp.driver_hwmons[2])
    description = DellS52xxCoreTemp.descriptions[2]

class DellS52xxTemp4(DellS52xxCoreTemp):
    name = 'Temp4'
    driver_hwmon = []
    driver_hwmon.append(DellS52xxCoreTemp.driver_hwmons[3])
    description = DellS52xxCoreTemp.descriptions[3]

class DellS52xxTemp5(DellS52xxCoreTemp):
    name = 'Temp5'
    driver_hwmon = []
    driver_hwmon.append(DellS52xxCoreTemp.driver_hwmons[4])
    description = DellS52xxCoreTemp.descriptions[4]

# Trident3 Sensors
class DellS52xxASICTemp(cumulus.sensors.Temp_Unit):
    driver_path = '/cumulus/switchd/run'
    description = cumulus.sensors.Temp_Unit.mac_die_sensor_name

class DellS52xxTemp6(DellS52xxASICTemp):
    name = 'Temp6'
    driver_hwmon = ['temp1']

class DellS52xxTemp7(DellS52xxASICTemp):
    name = 'Temp7'
    driver_hwmon = ['temp2']

class DellS52xxTemp8(DellS52xxASICTemp):
    name = 'Temp8'
    driver_hwmon = ['temp3']

class DellS52xxTemp9(DellS52xxASICTemp):
    name = 'Temp9'
    driver_hwmon = ['temp4']

class DellS52xxTemp10(DellS52xxASICTemp):
    name = 'Temp10'
    driver_hwmon = ['temp5']

class DellS52xxTemp11(DellS52xxASICTemp):
    name = 'Temp11'
    driver_hwmon = ['temp6']

class DellS52xxTemp12(DellS52xxASICTemp):
    name = 'Temp12'
    driver_hwmon = ['temp7']

class DellS52xxTemp13(DellS52xxASICTemp):
    name = 'Temp13'
    driver_hwmon = ['temp8']

class DellS52xxTemp14(DellS52xxASICTemp):
    name = 'Temp14'
    driver_hwmon = ['temp9']

class DellS52xxTemp15(DellS52xxASICTemp):
    name = 'Temp15'
    driver_hwmon = ['temp10']

class DellS52xxTemp16(DellS52xxASICTemp):
    name = 'Temp16'
    driver_hwmon = ['temp11']

class DellS52xxTemp17(DellS52xxASICTemp):
    name = 'Temp17'
    driver_hwmon = ['temp12']

# BMC Sensors
class DellS52xxBMCTemp(cumulus.sensors.Temp_Unit):
    driver_path = vhwmon_path + '/bmc:temp'

class DellS52xxTemp18(DellS52xxBMCTemp):
    name = 'Temp18'
    description = 'PT_Mid_temp'
    driver_hwmon = ['temp1']
    
class DellS52xxTemp19(DellS52xxBMCTemp):
    name = 'Temp19'
    description = 'NPU_Near_temp'
    driver_hwmon = ['temp2']
    
class DellS52xxTemp20(DellS52xxBMCTemp):
    name = 'Temp20'
    description = 'PT_Left_temp'
    driver_hwmon = ['temp3']

class DellS52xxTemp21(DellS52xxBMCTemp):
    name = 'Temp21'
    description = 'PT_Right_temp'
    driver_hwmon = ['temp4']
    
class DellS52xxTemp22(DellS52xxBMCTemp):
    name = 'Temp22'
    description = 'ILET_AF_temp'
    driver_hwmon = ['temp5']
    
class DellS52xxTemp23(DellS52xxBMCTemp):
    name = 'Temp23'
    description = 'CPU_temp'
    driver_hwmon = ['temp6']
    
# PSU temp sensors
class DellS52xxPSUTemp(cumulus.sensors.Temp_Unit):
    def detect_state_plat(self):
        if not self.psu.psu_ok:
            return 'ABSENT'
        return cumulus.sensors.Temp_Unit.detect_state_plat(self)

class DellS52xxTemp24(DellS52xxPSUTemp):
    driver_hwmon = ['temp1']
    name = 'PSU1Temp1'
    description = 'PSU1_AF_temp'
    psu = DellS52xxPSU1()
    driver_path = vhwmon_path + '/bmc:psu1'

class DellS52xxTemp25(DellS52xxPSUTemp):
    driver_hwmon = ['temp2']
    name = 'PSU1Temp2'
    description = 'PSU1_temp'
    psu = DellS52xxPSU1()
    driver_path = vhwmon_path + '/bmc:psu1'

class DellS52xxTemp26(DellS52xxPSUTemp):
    driver_hwmon = ['temp1']
    name = 'PSU2Temp1'
    description = 'PSU2_AF_temp'
    psu = DellS52xxPSU2()
    driver_path = vhwmon_path + '/bmc:psu2'

class DellS52xxTemp27(DellS52xxPSUTemp):
    driver_hwmon = ['temp2']
    name = 'PSU2Temp2'
    description = 'PSU2_temp'
    psu = DellS52xxPSU2()
    driver_path = vhwmon_path + '/bmc:psu2'

# Fans
class DellS52xxFan(cumulus.sensors.Fan_Unit):
    driver_path = vhwmon_path + '/bmc:fan'
    no_pwm = True
    fan_min = 1500
    fan_max = 23540

class DellS52xxFan1(DellS52xxFan):
    name = 'Fan1'
    description = 'FAN1_Rear_rpm'
    driver_hwmon = ['fan1']

class DellS52xxFan2(DellS52xxFan):
    name = 'Fan2'
    description = 'FAN2_Rear_rpm'
    driver_hwmon = ['fan2']

class DellS52xxFan3(DellS52xxFan):
    name = 'Fan3'
    description = 'FAN3_Rear_rpm'
    driver_hwmon = ['fan3']

class DellS52xxFan4(DellS52xxFan):
    name = 'Fan4'
    description = 'FAN4_Rear_rpm'
    driver_hwmon = ['fan4']

class DellS52xxFan5(DellS52xxFan):
    name = 'Fan5'
    description = 'FAN1_Front_rpm'
    driver_hwmon = ['fan5']

class DellS52xxFan6(DellS52xxFan):
    name = 'Fan6'
    description = 'FAN2_Front_rpm'
    driver_hwmon = ['fan6']

class DellS52xxFan7(DellS52xxFan):
    name = 'Fan7'
    description = 'FAN3_Front_rpm'
    driver_hwmon = ['fan7']

class DellS52xxFan8(DellS52xxFan):
    name = 'Fan8'
    description = 'FAN4_Front_rpm'
    driver_hwmon = ['fan8']

# PSU Fans
class DellS52xxPSUFan(cumulus.sensors.Fan_Unit):
    driver_hwmon = ['fan1']
    no_pwm = True

    def get_status(self):
        if not self.psu.psu_ok:
            return self.update_status(None, 'not_installed')
        return cumulus.sensors.Fan_Unit.get_status(self)

class DellS52xxPSU1Fan1(DellS52xxPSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan1'
    psu = DellS52xxPSU1()
    driver_path = vhwmon_path + "/bmc:psu1"

class DellS52xxPSU2Fan1(DellS52xxPSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan1'
    psu = DellS52xxPSU2()
    driver_path = vhwmon_path + "/bmc:psu2"

class DellS52xxSensors():
    units = (DellS52xxPSU1(),
             DellS52xxPSU2(),
             DellS52xxTemp1(),
             DellS52xxTemp2(),
             DellS52xxTemp3(),
             DellS52xxTemp4(),
             DellS52xxTemp5(),
             DellS52xxTemp6(),
             DellS52xxTemp7(),
             DellS52xxTemp8(),
             DellS52xxTemp9(),
             DellS52xxTemp10(),
             DellS52xxTemp11(),
             DellS52xxTemp12(),
             DellS52xxTemp13(),
             DellS52xxTemp14(),
             DellS52xxTemp15(),
             DellS52xxTemp16(),
             DellS52xxTemp17(),
             DellS52xxTemp18(),
             DellS52xxTemp19(),
             DellS52xxTemp20(),
             DellS52xxTemp21(),
             DellS52xxTemp22(),
             DellS52xxTemp23(),
             DellS52xxTemp24(),
             DellS52xxTemp25(),
             DellS52xxTemp26(),
             DellS52xxTemp27(),
             DellS52xxFan1(),
             DellS52xxFan2(),
             DellS52xxFan3(),
             DellS52xxFan4(),
             DellS52xxFan5(),
             DellS52xxFan6(),
             DellS52xxFan7(),
             DellS52xxFan8(),
             DellS52xxPSU1Fan1(),
             DellS52xxPSU2Fan1(),
             )

class DellS52xxSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_path = '/sys/devices/pci0000:00/0000:00:0c.0/0000:04:00.0'
    led_location = 'led_system'
    good_led_color = 'green'
    bad_led_color = 'amber'

class DellS52xxPowerLED(cumulus.sensors.StatusLED_Unit):
    name = 'Power'
    sources = ['PSU1', 'PSU2']
    led_path = '/sys/devices/pci0000:00/0000:00:0c.0/0000:04:00.0'
    led_location = 'led_power'
    good_led_color = 'green'
    bad_led_color = 'amber_blinking'

class DellS52xxLEDs():
    units = ( DellS52xxSystemLED(),
              DellS52xxPowerLED(),
              )

class DellS52xxBMC(cumulus.platform.BMC):
    name = 'bmc'
    type = 'ipmibmc'

