#
# Copyright (c) 2018 Cumulus Networks, Inc.  All Rights Reserved.
#

import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class DeltaAG9032V2QSFP28(cumulus.platform.TridentThree100GPort):

    def __init__(self, label, serdes, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.TridentThree100GPort.__init__(self,
                                                       label=label,
                                                       serdes=serdes,
                                                       leds=None,
                                                       phy_types=None,
                                                       phy_addrs=None,
                                                       phy_layer=sfp.type.upper(),
                                                       rx_lane_map=rx_lane_map,
                                                       rx_polarity_flip=rx_polarity_flip,
                                                       tx_lane_map=tx_lane_map,
                                                       tx_polarity_flip=tx_polarity_flip,
                                                       pre_emphasis=pre_emphasis,
                                                       oversub=True,
                                                       unit=0)
        self.sfp = sfp

class DeltaAG9032V2Switch(cumulus.platform.BCMSwitch):
    _qsfp28 = DeltaAG9032V2QSFP28
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _qsfp28(label="1",  serdes=(  0,   1,   2,   3), tx_lane_map=0x0213, rx_lane_map=0x3120, tx_polarity_flip=0x000c, rx_polarity_flip=0x0003),
        _qsfp28(label="2",  serdes=(  4,   5,   6,   7), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x000d, rx_polarity_flip=0x000c),
        _qsfp28(label="3",  serdes=(  8,   9,  10,  11), tx_lane_map=0x0213, rx_lane_map=0x3120, tx_polarity_flip=0x000c, rx_polarity_flip=0x0003),
        _qsfp28(label="4",  serdes=( 12,  13,  14,  15), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x000d, rx_polarity_flip=0x000c),

        _qsfp28(label="5",  serdes=( 16,  17,  18,  19), tx_lane_map=0x0213, rx_lane_map=0x3120, tx_polarity_flip=0x000c, rx_polarity_flip=0x0003),
        _qsfp28(label="6",  serdes=( 20,  21,  22,  23), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x000d, rx_polarity_flip=0x000c),
        _qsfp28(label="7",  serdes=( 24,  25,  26,  27), tx_lane_map=0x0213, rx_lane_map=0x3120, tx_polarity_flip=0x000c, rx_polarity_flip=0x000c),
        _qsfp28(label="8",  serdes=( 28,  29,  30,  31), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x000d, rx_polarity_flip=0x0003),

        _qsfp28(label="9",  serdes=( 32,  33,  34,  35), tx_lane_map=0x0213, rx_lane_map=0x3120, tx_polarity_flip=0x000c, rx_polarity_flip=0x000c),
        _qsfp28(label="10", serdes=( 36,  37,  38,  39), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x000d, rx_polarity_flip=0x0003),
        _qsfp28(label="11", serdes=( 40,  41,  42,  43), tx_lane_map=0x0213, rx_lane_map=0x3120, tx_polarity_flip=0x000c, rx_polarity_flip=0x000c),
        _qsfp28(label="12", serdes=( 44,  45,  46,  47), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x000d, rx_polarity_flip=0x0003),

        _qsfp28(label="13", serdes=( 48,  49,  50,  51), tx_lane_map=0x0213, rx_lane_map=0x3120, tx_polarity_flip=0x000c, rx_polarity_flip=0x000c),
        _qsfp28(label="14", serdes=( 52,  53,  54,  55), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x000d, rx_polarity_flip=0x0009),
        _qsfp28(label="15", serdes=( 56,  57,  58,  59), tx_lane_map=0x0213, rx_lane_map=0x3120, tx_polarity_flip=0x000c, rx_polarity_flip=0x0003),
        _qsfp28(label="16", serdes=( 60,  61,  62,  63), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x000d, rx_polarity_flip=0x0003),

        _qsfp28(label="17", serdes=( 64,  65,  66,  67), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x0003, rx_polarity_flip=0x000c),
        _qsfp28(label="18", serdes=( 68,  69,  70,  71), tx_lane_map=0x0213, rx_lane_map=0x3120, tx_polarity_flip=0x0003, rx_polarity_flip=0x0003),
        _qsfp28(label="19", serdes=( 72,  73,  74,  75), tx_lane_map=0x3210, rx_lane_map=0x2031, tx_polarity_flip=0x0005, rx_polarity_flip=0x000c),
        _qsfp28(label="20", serdes=( 76,  77,  78,  79), tx_lane_map=0x0213, rx_lane_map=0x3120, tx_polarity_flip=0x000c, rx_polarity_flip=0x000c),

        _qsfp28(label="21", serdes=( 80,  81,  82,  83), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x000c, rx_polarity_flip=0x0003),
        _qsfp28(label="22", serdes=( 84,  85,  86,  87), tx_lane_map=0x0213, rx_lane_map=0x3120, tx_polarity_flip=0x000c, rx_polarity_flip=0x000c),
        _qsfp28(label="23", serdes=( 88,  89,  90,  91), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x000c, rx_polarity_flip=0x0003),
        _qsfp28(label="24", serdes=( 92,  93,  94,  95), tx_lane_map=0x0213, rx_lane_map=0x3120, tx_polarity_flip=0x000c, rx_polarity_flip=0x000c),

        _qsfp28(label="25", serdes=( 96,  97,  98,  99), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x000c, rx_polarity_flip=0x0003),
        _qsfp28(label="26", serdes=(100, 101, 102, 103), tx_lane_map=0x0213, rx_lane_map=0x3120, tx_polarity_flip=0x000c, rx_polarity_flip=0x000c),
        _qsfp28(label="27", serdes=(104, 105, 106, 107), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x000c, rx_polarity_flip=0x000c),
        _qsfp28(label="28", serdes=(108, 109, 110, 111), tx_lane_map=0x0213, rx_lane_map=0x3120, tx_polarity_flip=0x000c, rx_polarity_flip=0x0003),

        _qsfp28(label="29", serdes=(112, 113, 114, 115), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x000c, rx_polarity_flip=0x000c),
        _qsfp28(label="30", serdes=(116, 117, 118, 119), tx_lane_map=0x0213, rx_lane_map=0x3120, tx_polarity_flip=0x000c, rx_polarity_flip=0x0003),
        _qsfp28(label="31", serdes=(120, 121, 122, 123), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0x000c, rx_polarity_flip=0x000c),
        _qsfp28(label="32", serdes=(124, 125, 126, 127), tx_lane_map=0x0213, rx_lane_map=0x3120, tx_polarity_flip=0x000c, rx_polarity_flip=0x0003),
    )

vhwmon_path = '/sys/class/misc/vhwmon'

class DeltaAG9032V2CPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/platform/delta_ag9032v2_cpld.0'

# Intel Denverton CPU
class DeltaAG9032V2CoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    no_shutdown = True

class DeltaAG9032V2CPUTemp1(DeltaAG9032V2CoreTemp):
    name = 'Temp1'
    description = 'Physical id 0'
    driver_hwmon = ['temp1']

class DeltaAG9032V2CPUTemp2(DeltaAG9032V2CoreTemp):
    name = 'Temp2'
    description = 'Core 2'
    driver_hwmon = ['temp4']

class DeltaAG9032V2CPUTemp3(DeltaAG9032V2CoreTemp):
    name = 'Temp3'
    description = 'Core 6'
    driver_hwmon = ['temp8']

class DeltaAG9032V2CPUTemp4(DeltaAG9032V2CoreTemp):
    name = 'Temp4'
    description = 'Core 8'
    driver_hwmon = ['temp10']

class DeltaAG9032V2CPUTemp5(DeltaAG9032V2CoreTemp):
    name = 'Temp5'
    description = 'Core 12'
    driver_hwmon = ['temp14']

# Trident3 Sensors TBD

# BMC Sensors
class DeltaAG9032V2BMCTemp(cumulus.sensors.Temp_Unit):
    driver_path = vhwmon_path + '/bmc:temp'

class DeltaAG9032V2BMCTemp1(DeltaAG9032V2BMCTemp):
    name = 'Temp6'
    description = 'Sensor_Temp_1'
    driver_hwmon = ['temp1']

class DeltaAG9032V2BMCTemp2(DeltaAG9032V2BMCTemp):
    name = 'Temp7'
    description = 'Sensor_Temp_2'
    driver_hwmon = ['temp2']

class DeltaAG9032V2BMCTemp3(DeltaAG9032V2BMCTemp):
    name = 'Temp8'
    description = 'Sensor_Temp_3'
    driver_hwmon = ['temp3']

class DeltaAG9032V2BMCTemp4(DeltaAG9032V2BMCTemp):
    name = 'Temp9'
    description = 'Sensor_Temp_4'
    driver_hwmon = ['temp4']

class DeltaAG9032V2BMCTemp5(DeltaAG9032V2BMCTemp):
    name = 'Temp10'
    description = 'Sensor_Temp_5'
    driver_hwmon = ['temp5']

# Fans
class DeltaAG9032V2Fan(cumulus.sensors.Fan_Unit):
    driver_path = vhwmon_path + '/bmc:fan'
    no_pwm = True

class DeltaAG9032V2Fan1(DeltaAG9032V2Fan):
    name = 'Fan1'
    description = 'Fan_Tray_1'
    driver_hwmon = ['fan1']

class DeltaAG9032V2Fan2(DeltaAG9032V2Fan):
    name = 'Fan2'
    description = 'Fan_Tray_2'
    driver_hwmon = ['fan2']

class DeltaAG9032V2Fan3(DeltaAG9032V2Fan):
    name = 'Fan3'
    description = 'Fan_Tray_3'
    driver_hwmon = ['fan3']

class DeltaAG9032V2Fan4(DeltaAG9032V2Fan):
    name = 'Fan4'
    description = 'Fan_Tray_4'
    driver_hwmon = ['fan4']

class DeltaAG9032V2Fan5(DeltaAG9032V2Fan):
    name = 'Fan5'
    description = 'Fan_Tray_5'
    driver_hwmon = ['fan5']

# PSUs
class DeltaAG9032V2PSU1(cumulus.sensors.PSU_Unit):
    name = 'PSU1'
    driver_path = DeltaAG9032V2CPLD.path
    driver_hwmon = ['psu_pwr1']

    def get_status(self):
        val = self.attrs.get('psu_pwr1_all_ok', 0)
        self.attrs['psu_pwr1_all_ok'] = val
        DeltaAG9032V2PSU1.psu_ok = int(val)
        return cumulus.sensors.PSU_Unit.get_status(self)

class DeltaAG9032V2PSU2(cumulus.sensors.PSU_Unit):
    name = 'PSU2'
    driver_path = DeltaAG9032V2CPLD.path
    driver_hwmon = ['psu_pwr2']

    def get_status(self):
        val = self.attrs.get('psu_pwr2_all_ok', 0)
        self.attrs['psu_pwr2_all_ok'] = val
        DeltaAG9032V2PSU2.psu_ok = int(val)
        return cumulus.sensors.PSU_Unit.get_status(self)
                                    
# PSU Temps
class DeltaAG9032V2PSUTemp(cumulus.sensors.Temp_Unit):
    driver_hwmon = ['temp1']

    def find_and_add_attrs(self):
        if ((self.get_cpld_value(self.temp_all_ok) == '1')):
            return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)
        else:
            return -1

class DeltaAG9032V2PSU1Temp1(DeltaAG9032V2PSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1_Temp_1'
    psu = DeltaAG9032V2PSU1()
    driver_path = vhwmon_path + '/bmc:psu1'
    temp_all_ok = 'psu_pwr1_all_ok'

class DeltaAG9032V2PSU1Temp2(DeltaAG9032V2PSUTemp):
    name = 'PSU1Temp2'
    description = 'PSU1_Temp_2'
    psu = DeltaAG9032V2PSU1()
    driver_path = vhwmon_path + '/bmc:psu1'
    temp_all_ok = 'psu_pwr1_all_ok'

class DeltaAG9032V2PSU2Temp1(DeltaAG9032V2PSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2_Temp_1'
    psu = DeltaAG9032V2PSU2()
    driver_path = vhwmon_path + '/bmc:psu2'
    temp_all_ok = 'psu_pwr2_all_ok'

class DeltaAG9032V2PSU2Temp2(DeltaAG9032V2PSUTemp):
    name = 'PSU2Temp2'
    description = 'PSU2_Temp_2'
    psu = DeltaAG9032V2PSU2()
    driver_path = vhwmon_path + '/bmc:psu2'
    temp_all_ok = 'psu_pwr2_all_ok'

# PSU Fans
class DeltaAG9032V2PSUFan(cumulus.sensors.Fan_Unit):
    driver_hwmon = ['fan1']
    no_pwm = True

    def get_status(self):
        if not self.psu.psu_ok:
            return self.update_status(None, 'not_installed')
        return cumulus.sensors.Fan_Unit.get_status(self)

class DeltaAG9032V2PSU1Fan1(DeltaAG9032V2PSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1_Fan'
    psu = DeltaAG9032V2PSU1()
    driver_path = vhwmon_path + '/bmc:psu1'

class DeltaAG9032V2PSU2Fan1(DeltaAG9032V2PSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2_Fan'
    psu = DeltaAG9032V2PSU2()
    driver_path = vhwmon_path + '/bmc:psu2'

# Sensors
class DeltaAG9032V2Sensors():
    units = (DeltaAG9032V2CPUTemp1(),
             DeltaAG9032V2CPUTemp2(),
             DeltaAG9032V2CPUTemp3(),
             DeltaAG9032V2CPUTemp4(),
             DeltaAG9032V2CPUTemp5(),
             DeltaAG9032V2BMCTemp1(),
             DeltaAG9032V2BMCTemp2(),
             DeltaAG9032V2BMCTemp3(),
             DeltaAG9032V2BMCTemp4(),
             DeltaAG9032V2BMCTemp5(),
             DeltaAG9032V2Fan1(),
             DeltaAG9032V2Fan2(),
             DeltaAG9032V2Fan3(),
             DeltaAG9032V2Fan4(),
             DeltaAG9032V2Fan5(),
             DeltaAG9032V2PSU1(),
             DeltaAG9032V2PSU2(),
             DeltaAG9032V2PSU1Temp1(),
             DeltaAG9032V2PSU1Temp2(),
             DeltaAG9032V2PSU2Temp1(),
             DeltaAG9032V2PSU2Temp2(),
             DeltaAG9032V2PSU1Fan1(),
             DeltaAG9032V2PSU2Fan1(),
             )

# System LEDs are managed by the hardware

class DeltaAG9032V2BMC(cumulus.platform.BMC):
    name = 'bmc'
    type = 'ipmibmc'

class DeltaAG9032V2(cumulus.platform.Platform):
    name = 'delta,ag9032v2'
    chip = cumulus.platform.TridentThreeX7_56870_Chip()
    switch = DeltaAG9032V2Switch(chip)
    bmc = DeltaAG9032V2BMC()
    cpld = DeltaAG9032V2CPLD()
    sensors = DeltaAG9032V2Sensors()
    statusleds = None
    pwms = None

Platform = DeltaAG9032V2
