#
# Copyright (c) 2017 Cumulus Networks, Inc.  All Rights Reserved.
#

import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class QuantaIX7QSFP28(cumulus.platform.TridentThree100GPort):

    def __init__(self, label, serdes, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.TridentThree100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   leds=None,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

class QuantaIX7Switch(cumulus.platform.BCMSwitch):
    _qsfp28 = QuantaIX7QSFP28
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _qsfp28(label="1",  serdes=( 28,  29,  30,  31), tx_lane_map=0x1230, rx_lane_map=0x1302, tx_polarity_flip=0x000d, rx_polarity_flip=0x000c,),
        _qsfp28(label="2",  serdes=( 32,  33,  34,  35), tx_lane_map=0x3210, rx_lane_map=0x0123, tx_polarity_flip=0x000f, rx_polarity_flip=0x000a,),
        _qsfp28(label="3",  serdes=( 40,  41,  42,  43), tx_lane_map=0x0213, rx_lane_map=0x1302, tx_polarity_flip=0x0006, rx_polarity_flip=0x0003,),
        _qsfp28(label="4",  serdes=( 44,  45,  46,  47), tx_lane_map=0x3210, rx_lane_map=0x2103, tx_polarity_flip=0x000f, rx_polarity_flip=0x0004,),

        _qsfp28(label="5",  serdes=(  0,   1,   2,   3), tx_lane_map=0x0213, rx_lane_map=0x1302, tx_polarity_flip=0x0006, rx_polarity_flip=0x000c,),
        _qsfp28(label="6",  serdes=(  4,   5,   6,   7), tx_lane_map=0x3210, rx_lane_map=0x2103, tx_polarity_flip=0x000f, rx_polarity_flip=0x000b,),
        _qsfp28(label="7",  serdes=(  8,   9,  10,  11), tx_lane_map=0x0213, rx_lane_map=0x1302, tx_polarity_flip=0x0006, rx_polarity_flip=0x000e,),
        _qsfp28(label="8",  serdes=( 12,  13,  14,  15), tx_lane_map=0x3210, rx_lane_map=0x2031, tx_polarity_flip=0x000f, rx_polarity_flip=0x000d,),

        _qsfp28(label="9",  serdes=( 16,  17,  18,  19), tx_lane_map=0x0123, rx_lane_map=0x1302, tx_polarity_flip=0x000c, rx_polarity_flip=0x000c,),
        _qsfp28(label="10", serdes=( 20,  21,  22,  23), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=0x000f, rx_polarity_flip=0x0006,),
        _qsfp28(label="11", serdes=( 24,  25,  26,  27), tx_lane_map=0x0213, rx_lane_map=0x1302, tx_polarity_flip=0x0009, rx_polarity_flip=0x0003,),
        _qsfp28(label="12", serdes=( 36,  37,  38,  39), tx_lane_map=0x3210, rx_lane_map=0x2103, tx_polarity_flip=0x000f, rx_polarity_flip=0x0004,),

        _qsfp28(label="13", serdes=( 48,  49,  50,  51), tx_lane_map=0x0213, rx_lane_map=0x1302, tx_polarity_flip=0x0009, rx_polarity_flip=0x0003,),
        _qsfp28(label="14", serdes=( 52,  53,  54,  55), tx_lane_map=0x3210, rx_lane_map=0x2103, tx_polarity_flip=0x0000, rx_polarity_flip=0x0004,),
        _qsfp28(label="15", serdes=( 56,  57,  58,  59), tx_lane_map=0x0213, rx_lane_map=0x1302, tx_polarity_flip=0x0006, rx_polarity_flip=0x0003,),
        _qsfp28(label="16", serdes=( 60,  61,  62,  63), tx_lane_map=0x3210, rx_lane_map=0x2103, tx_polarity_flip=0x000f, rx_polarity_flip=0x0004,),

        _qsfp28(label="17", serdes=( 64,  65,  66,  67), tx_lane_map=0x3210, rx_lane_map=0x3120, tx_polarity_flip=0x000c, rx_polarity_flip=0x000c,),
        _qsfp28(label="18", serdes=( 68,  69,  70,  71), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=0x0000, rx_polarity_flip=0x0001,),
        _qsfp28(label="19", serdes=( 72,  73,  74,  75), tx_lane_map=0x3120, rx_lane_map=0x3120, tx_polarity_flip=0x0009, rx_polarity_flip=0x000c,),
        _qsfp28(label="20", serdes=( 76,  77,  78,  79), tx_lane_map=0x0213, rx_lane_map=0x0321, tx_polarity_flip=0x0009, rx_polarity_flip=0x000b,),

        _qsfp28(label="21", serdes=( 92,  93,  94,  95), tx_lane_map=0x3120, rx_lane_map=0x3120, tx_polarity_flip=0x0009, rx_polarity_flip=0x0003,),
        _qsfp28(label="22", serdes=(100, 101, 102, 103), tx_lane_map=0x0321, rx_lane_map=0x0321, tx_polarity_flip=0x000e, rx_polarity_flip=0x000b,),
        _qsfp28(label="23", serdes=(104, 105, 106, 107), tx_lane_map=0x3120, rx_lane_map=0x3120, tx_polarity_flip=0x0009, rx_polarity_flip=0x0003,),
        _qsfp28(label="24", serdes=(108, 109, 110, 111), tx_lane_map=0x0123, rx_lane_map=0x0321, tx_polarity_flip=0x000f, rx_polarity_flip=0x0004,),

        _qsfp28(label="25", serdes=(112, 113, 114, 115), tx_lane_map=0x0312, rx_lane_map=0x3120, tx_polarity_flip=0x0004, rx_polarity_flip=0x000b,),
        _qsfp28(label="26", serdes=(116, 117, 118, 119), tx_lane_map=0x0123, rx_lane_map=0x1320, tx_polarity_flip=0x000f, rx_polarity_flip=0x0007,),
        _qsfp28(label="27", serdes=(120, 121, 122, 123), tx_lane_map=0x3120, rx_lane_map=0x0123, tx_polarity_flip=0x0009, rx_polarity_flip=0x000a,),
        _qsfp28(label="28", serdes=(124, 125, 126, 127), tx_lane_map=0x0123, rx_lane_map=0x0321, tx_polarity_flip=0x0000, rx_polarity_flip=0x0004,),

        _qsfp28(label="29", serdes=( 80,  81,  82,  83), tx_lane_map=0x3201, rx_lane_map=0x3120, tx_polarity_flip=0x000e, rx_polarity_flip=0x0004,),
        _qsfp28(label="30", serdes=( 84,  85,  86,  87), tx_lane_map=0x0123, rx_lane_map=0x0321, tx_polarity_flip=0x000f, rx_polarity_flip=0x000b,),
        _qsfp28(label="31", serdes=( 88,  89,  90,  91), tx_lane_map=0x3210, rx_lane_map=0x0123, tx_polarity_flip=0x000f, rx_polarity_flip=0x0005,),
        _qsfp28(label="32", serdes=( 96,  97,  98,  99), tx_lane_map=0x0312, rx_lane_map=0x0321, tx_polarity_flip=0x000c, rx_polarity_flip=0x0005,),
    )

vhwmon_path = '/sys/class/misc/vhwmon'

# Intel Rangeley CPU
class QuantaIX7CoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    no_shutdown = True

class QuantaIX7CPUTemp1(QuantaIX7CoreTemp):
    name = 'Temp1'
    description = 'Core 0'
    driver_hwmon = ['temp2']

class QuantaIX7CPUTemp2(QuantaIX7CoreTemp):
    name = 'Temp2'
    description = 'Core 1'
    driver_hwmon = ['temp3']

class QuantaIX7CPUTemp3(QuantaIX7CoreTemp):
    name = 'Temp3'
    description = 'Core 2'
    driver_hwmon = ['temp4']

class QuantaIX7CPUTemp4(QuantaIX7CoreTemp):
    name = 'Temp4'
    description = 'Core 3'
    driver_hwmon = ['temp5']

# Trident3 Sensors
class QuantaIX7ASICTemp(cumulus.sensors.Temp_Unit):
    driver_path = '/cumulus/switchd/run'
    description = cumulus.sensors.Temp_Unit.mac_die_sensor_name

class QuantaIX7ASICTemp1(QuantaIX7ASICTemp):
    name = 'Temp5'
    driver_hwmon = ['temp1']

class QuantaIX7ASICTemp2(QuantaIX7ASICTemp):
    name = 'Temp6'
    driver_hwmon = ['temp2']

class QuantaIX7ASICTemp3(QuantaIX7ASICTemp):
    name = 'Temp7'
    driver_hwmon = ['temp3']

class QuantaIX7ASICTemp4(QuantaIX7ASICTemp):
    name = 'Temp8'
    driver_hwmon = ['temp4']

class QuantaIX7ASICTemp5(QuantaIX7ASICTemp):
    name = 'Temp9'
    driver_hwmon = ['temp5']

class QuantaIX7ASICTemp6(QuantaIX7ASICTemp):
    name = 'Temp10'
    driver_hwmon = ['temp6']

class QuantaIX7ASICTemp7(QuantaIX7ASICTemp):
    name = 'Temp11'
    driver_hwmon = ['temp7']

class QuantaIX7ASICTemp8(QuantaIX7ASICTemp):
    name = 'Temp12'
    driver_hwmon = ['temp8']

class QuantaIX7ASICTemp9(QuantaIX7ASICTemp):
    name = 'Temp13'
    driver_hwmon = ['temp9']

class QuantaIX7ASICTemp10(QuantaIX7ASICTemp):
    name = 'Temp14'
    driver_hwmon = ['temp10']

class QuantaIX7ASICTemp11(QuantaIX7ASICTemp):
    name = 'Temp15'
    driver_hwmon = ['temp11']

class QuantaIX7ASICTemp12(QuantaIX7ASICTemp):
    name = 'Temp16'
    driver_hwmon = ['temp12']

# BMC Sensors
class QuantaIX7BMCTemp(cumulus.sensors.Temp_Unit):
    driver_path = vhwmon_path + '/bmc:temp'

class QuantaIX7BMCTemp1(QuantaIX7BMCTemp):
    name = 'Temp17'
    description = 'Temp_Ambient_0'
    driver_hwmon = ['temp1']

class QuantaIX7BMCTemp2(QuantaIX7BMCTemp):
    name = 'Temp18'
    description = 'Temp_Ambient_1'
    driver_hwmon = ['temp2']

class QuantaIX7BMCTemp3(QuantaIX7BMCTemp):
    name = 'Temp19'
    description = 'Temp_Ambient_2'
    driver_hwmon = ['temp3']

class QuantaIX7BMCTemp4(QuantaIX7BMCTemp):
    name = 'Temp20'
    description = 'Temp_Ambient_3'
    driver_hwmon = ['temp4']

class QuantaIX7BMCTemp5(QuantaIX7BMCTemp):
    name = 'Temp21'
    description = 'Temp_Ambient_4'
    driver_hwmon = ['temp5']

class QuantaIX7BMCTemp6(QuantaIX7BMCTemp):
    name = 'Temp22'
    description = 'Temp_Ambient_5'
    driver_hwmon = ['temp6']

class QuantaIX7BMCTemp7(QuantaIX7BMCTemp):
    name = 'Temp23'
    description = 'Temp_Ambient_6'
    driver_hwmon = ['temp7']

class QuantaIX7BMCTemp8(QuantaIX7BMCTemp):
    name = 'Temp24'
    description = 'Temp_CPU'
    driver_hwmon = ['temp8']

# Fans
class QuantaIX7Fan(cumulus.sensors.Fan_Unit):
    driver_path = vhwmon_path + '/bmc:fan'
    no_pwm = True

class QuantaIX7Fan1(QuantaIX7Fan):
    name = 'Fan1'
    description = 'Fan 1_1'
    driver_hwmon = ['fan1']

class QuantaIX7Fan2(QuantaIX7Fan):
    name = 'Fan2'
    description = 'Fan 1_2'
    driver_hwmon = ['fan2']

class QuantaIX7Fan3(QuantaIX7Fan):
    name = 'Fan3'
    description = 'Fan 2_1'
    driver_hwmon = ['fan3']

class QuantaIX7Fan4(QuantaIX7Fan):
    name = 'Fan4'
    description = 'Fan 2_2'
    driver_hwmon = ['fan4']

class QuantaIX7Fan5(QuantaIX7Fan):
    name = 'Fan5'
    description = 'Fan 3_1'
    driver_hwmon = ['fan5']

class QuantaIX7Fan6(QuantaIX7Fan):
    name = 'Fan6'
    description = 'Fan 3_2'
    driver_hwmon = ['fan6']

class QuantaIX7Fan7(QuantaIX7Fan):
    name = 'Fan7'
    description = 'Fan 4_1'
    driver_hwmon = ['fan7']

class QuantaIX7Fan8(QuantaIX7Fan):
    name = 'Fan8'
    description = 'Fan 4_2'
    driver_hwmon = ['fan8']

class QuantaIX7Fan9(QuantaIX7Fan):
    name = 'Fan9'
    description = 'Fan 5_1'
    driver_hwmon = ['fan9']

class QuantaIX7Fan10(QuantaIX7Fan):
    name = 'Fan10'
    description = 'Fan 5_2'
    driver_hwmon = ['fan10']

class QuantaIX7Fan11(QuantaIX7Fan):
    name = 'Fan11'
    description = 'Fan 6_1'
    driver_hwmon = ['fan11']

class QuantaIX7Fan12(QuantaIX7Fan):
    name = 'Fan12'
    description = 'Fan 6_2'
    driver_hwmon = ['fan12']

#PSUs
class QuantaIX7PSU(cumulus.sensors.PSU_Unit):
    driver_hwmon = ['power1']

class QuantaIX7PSU1(QuantaIX7PSU):
    name = 'PSU1'
    driver_path = vhwmon_path + '/bmc:psu1'
    psu_ok = 0

    def get_status(self):
        val = self.attrs.get('power1_input', 0)
        if val != 0:
            val = 1
        self.attrs['psu_pwr1_all_ok'] = val
        QuantaIX7PSU1.psu_ok = val
        return cumulus.sensors.PSU_Unit.get_status(self)

class QuantaIX7PSU2(QuantaIX7PSU):
    name = 'PSU2'
    driver_path = vhwmon_path + '/bmc:psu2'
    psu_ok = 0

    def get_status(self):
        val = self.attrs.get('power1_input', 0)
        if val != 0:
            val = 1
        self.attrs['psu_pwr2_all_ok'] = val
        QuantaIX7PSU2.psu_ok = val
        return cumulus.sensors.PSU_Unit.get_status(self)

class QuantaIX7PSUFan(cumulus.sensors.Fan_Unit):
    driver_hwmon = ['fan1']
    no_pwm = True

    def get_status(self):
        if not self.psu.psu_ok:
            return self.update_status(None, 'not_installed')
        return cumulus.sensors.Fan_Unit.get_status(self)

class QuantaIX7PSU1Fan1(QuantaIX7PSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan1'
    psu = QuantaIX7PSU1()
    driver_path = psu.driver_path

class QuantaIX7PSU2Fan1(QuantaIX7PSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan1'
    psu = QuantaIX7PSU2()
    driver_path = psu.driver_path

class QuantaIX7Sensors():
    units = (QuantaIX7CPUTemp1(),
             QuantaIX7CPUTemp2(),
             QuantaIX7CPUTemp3(),
             QuantaIX7CPUTemp4(),
             QuantaIX7ASICTemp1(),
             QuantaIX7ASICTemp2(),
             QuantaIX7ASICTemp3(),
             QuantaIX7ASICTemp4(),
             QuantaIX7ASICTemp5(),
             QuantaIX7ASICTemp6(),
             QuantaIX7ASICTemp7(),
             QuantaIX7ASICTemp8(),
             QuantaIX7ASICTemp9(),
             QuantaIX7ASICTemp10(),
             QuantaIX7ASICTemp11(),
             QuantaIX7ASICTemp12(),
             QuantaIX7BMCTemp1(),
             QuantaIX7BMCTemp2(),
             QuantaIX7BMCTemp3(),
             QuantaIX7BMCTemp4(),
             QuantaIX7BMCTemp5(),
             QuantaIX7BMCTemp6(),
             QuantaIX7BMCTemp7(),
             QuantaIX7BMCTemp8(),
             QuantaIX7Fan1(),
             QuantaIX7Fan2(),
             QuantaIX7Fan3(),
             QuantaIX7Fan4(),
             QuantaIX7Fan5(),
             QuantaIX7Fan6(),
             QuantaIX7Fan7(),
             QuantaIX7Fan8(),
             QuantaIX7Fan9(),
             QuantaIX7Fan10(),
             QuantaIX7Fan11(),
             QuantaIX7Fan12(),
             QuantaIX7PSU1(),
             QuantaIX7PSU2(),
             QuantaIX7PSU1Fan1(),
             QuantaIX7PSU2Fan1(),
             )

class QuantaIX7BMC(cumulus.platform.BMC):
    name = 'bmc'
    type = 'ipmibmc'

class QuantaIX7(cumulus.platform.Platform):
    name = 'quanta,ix7_rglbmc'
    chip = cumulus.platform.TridentThreeX7_56870_Chip()
    switch = QuantaIX7Switch(chip)
    bmc = QuantaIX7BMC()
    sensors = QuantaIX7Sensors()
    statusleds = None
    pwms = None

Platform = QuantaIX7
