#! /usr/bin/python
# Copyright 2015 Cumulus Networks, all rights reserved

#############################################################################
# This is a base class to read and display PSU and Fan Info when it is
# exported in the form of sysfs files.
#

try:
    import os
except ImportError, e:
    raise ImportError (str(e) + "- required module not found")


class SysFsDecoder():
    def __init__(self, name, path, cpld_root, ro):
            self.name = name
            self.path = path
            self.cpld_root = cpld_root
            self.ro = ro
            self.fmt = {    'county' : 'Country' ,
                            'part_num': 'Part Number',
                            'mfg_id'  : 'Manufacturing ID',
                            'mfg_date' : 'Manufacturing Date',
                            'serial_num' : 'Serial Number',
                            'service_tag' : 'Service Tag',
                            'label_rev' : 'Label Revision',
                        }
                
    def check_status(self):
        fname = ''
        status = ''
        # Expecting psu1, psu2 and fan[0-9]*
        if self.name.startswith('fan'):
            fname = self.name + '_all_ok'
        elif self.name == 'psu1':
            fname = 'psu_pwr1_all_ok'
        elif self.name == 'psu2':
            fname = 'psu_pwr2_all_ok'

        path = os.path.join(self.path, fname)
        with open(path) as fp:
            status = fp.read().rsplit('\n')[0]
        if status == '1':
            return 'ok'
        else:
            return 'not ok'

    def read_eeprom(self):
        e = {}
        
        for key, item in self.fmt.iteritems():
            fname = self.name + '_' + key
            path = os.path.join(self.path, fname)
            if os.path.exists(path):
                with open(path) as fp:
                    e[item] = fp.read().rsplit('\n')[0]
        return e
                
    def decode_eeprom(self, e):
        for key, item in e.iteritems():
            print ("%-20s:  %s" %(key, item))
        
    def is_checksum_valid(self, e):
        return (None, None)

class PMBUSDecoder(SysFsDecoder):
    def __init__(self, name, path, cpld_root, ro):
            self.name = name
            self.path = path
            self.cpld_root = cpld_root
            self.ro = ro
            self.fmt = {'mfr_id' : 'ID' ,
                        'mfr_model' : 'Model' , 
                        'mfr_revision' : 'Revision' ,
                        'mfr_location' : 'Location' ,
                        'mfr_serial' : 'Serial Number' ,        
                    }
            
    def read_eeprom(self):
        e = {}
        
        for key, item in self.fmt.iteritems():
            path = os.path.join(self.path, key)
            if os.path.exists(path):
                with open(path) as fp:
                    e[item] = fp.read().rsplit('\n')[0]
        return e
            
